package edu.neu.ccs.com3205.lodchecker;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jdt.core.*;
import org.eclipse.ui.*;

import org.eclipse.ajdt.ui.AspectJPlugin;

import java.io.*;
import java.util.*;
import java.net.*;

import edu.neu.ccs.com3205.lodchecker.utils.*;
import edu.neu.ccs.com3205.lodchecker.ui.*;

/**
 * Checker.java
 * This class contains methods involved with enabling and disabling the 
 * LoDChecker for a particular project.  Enabling and Disabling of the checker 
 * can be done by simply calling the corresponding enable and disable functions.
 * Enable and Disable call many of the other functions contained in this class.
 * The other functions here have been made public as there are points in
 * the execution of the IDE where it is ideal to verify that certain steps 
 * have not been undone by the user, i.e. the existance of the log4j configuration
 * file is verified before a launch by calling the addLog4jLCF prior to a launch, etc.
 * 
 * @author Paul Freeman
 */
public class Checker {
	
  private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();
	public static final String AJ_NATURE = AspectJPlugin.ID_NATURE;

	// AnyDynamic property constants
	public static final String ANYDYNAMIC_FILE = "AnyDynamic.java";
	public static final String ANYD_TOP = "package lawOfDemeter;\n" + 
											"abstract class AnyDynamic{";
	public static final String ANYD_POINTCUT = " pointcut ";
	public static final String ANYD_TOCHECK = " ToCheck(): !ToAvoid() && (Any.MethodCall() || Any.Get());";
	public static final String ANYD_TOAVOID = " ToAvoid() ";
	public static final String ANYD_GLOBAL = " GlobalPreferred() ";
	public static final String ANYD_STABLE = " Stable() ";
	public static final String ANY_STATIC_CALL = " Any.StaticCall() ";
	public static final String ANY_STATIC_GET = " Any.StaticGet() ";
	public static final String ANY_JAVA_CALL = " Any.JavaPackageCall() ";
	public static final String ANY_JAVA_GET = " Any.JavaPackageGet() ";
	public static final String ANY_NOTHING = " Any.Nothing() ";

	// file names
	public static final String LCF = "LoDLogger.lcf";								// log configuration file
	public static final String LST_FILE = "LoDCheckerBuild.lst";			// build lst file
  public static final String TEMP_VFILE_NAME = "Last_LoDViolations.log";// stores the last session's violations
	public static final String PERSIST_VFILE_NAME = "LoDChecker.log"; // stores the violations from multiple runs - generated by Checker aspect Logger.java - name is maintianed in the LoDChecker.lcf @todo read in from file
	
	// directories
	public static final String CHECKER_SRC_DIR = "LoDChecker";
	public static final String OBJECT_FORM_DIR = "ObjectForm";
	public static final String CLASS_FORM_DIR = "ClassForm";
	public static final String CHECKER_PACKAGE = "lawOfDemeter";
	
	// Strings
	public static final String V_TAG = "!! LoD violation:";		// tag beginning all violation statements in a violation log file

	// stores the current lst file for aspectJ builds
	private static URL buildFile;
	
	/**
	 * Does the following if necessary:<br>
	 *  add the required libraries to the classpath of this project<br>
	 * 	add the required natures to this project<br>
	 *  add the log configuration file to the project directory<br>
	 *  creates and adds the custom pointcut file based on the values stored for this project<br>
	 *  displays the views used by the checker<br>
	 * 
	 * @return Returns true if all of the above were either not necessary or were
	 * successfully accomplished, returns false if any were tried and failed.
	 */
	synchronized public static void enable(IProject project) throws CoreException {
		plugin.log("Checker.enable called");
		
		addRequiredLibs(project);		// do this before adding natures to avoid unnecessary builds with added builders
		addRequiredNatures(project);
		addLog4JLCF(project);
		createCustomPointcutFile(project);
		showViews();
	}
	
	/**
	 * Does the following if necessary:<br>
	 * 	removes the required natures from this project<br>
	 *  removes the libraries required by the checker from the classpath of this project<br>
	 *  removes the log configuration file from the project directory<br>
	 * 
	 * @return Returns true if all of the above were either not necessary or were
	 * successfully accomplished, returns false if any were tried and failed.
	 */
	synchronized public static void disable(IProject project) throws CoreException {
		disable(project, false);
	}

	/**
	 * Does the following if necessary:<br>
	 * 	removes the required natures from this project - disables the AspectJNature 
	 * iff removeAJNature is true<br>
	 *  removes the libraries required by the checker from the classpath of this project<br>
	 *  removes the log configuration file from the project directory<br>
	 * 
	 * @return Returns true if all of the above were either not necessary or were
	 * successfully accomplished, returns false if any were tried and failed.
	 */
	synchronized public static void disable(IProject project, boolean removeAJNature) throws CoreException {
		plugin.log("Checker.disable called");
		
		removeRequiredNatures(project, !removeAJNature);
		removeRequiredLibs(project);	// do this after removing natures to avoid unnecessary builds with added builders
		removeLog4JLCF(project);			// ?? does not need to be successfull		
//		hideViews();  -- don't want to hide since there may be other projects open which enable the checker
	}
	
	/**
	 * @return Returns true if the checker has been enabled for this project and false otherwise
	 */
	synchronized public static boolean isEnabled(IProject project){
		return CheckerProperties.retrieveSettingBoolean(
																project,
																CheckerProperties.ENABLE_PROPERTY,
																false);
	}
	
	/**
	 * Add the log configuration file to the project if it does not already exist.
	 * If it exists, this function will not overwrite the existing file as the user
	 * may have modified it to suite his/her needs... although this may cause problems
	 * with loading violations into the display... perhaps it should be overwritten.<br>
	 * 
	 * This function is called when the checker is enabled for a project, as well as 
	 * when a launch of a project occurs.
	 */
	synchronized public static void addLog4JLCF(IProject project) throws CoreException {
		plugin.log("adding lcf to project " + project);
		
		try{
			File newLcf = project.getLocation().append(LCF).toFile();
			
			if(!newLcf.exists()){
				// create newLcf by copying the masterLcf
				File masterLcf = new File(LoDCheckerPlugin.getAbsolutePath() + File.separator + LCF);
				Files.copyFile(masterLcf, newLcf);
			}else{
				plugin.log("lcf already existed - not added");
			}
		}catch(Exception e){
			String message = "Failed to create lcf for project";
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
		}		
	}

	/**
	 * removes the log configuration file from the project
	 * called when the checker is disabled
	 * @return returns true if the file doesn't exist or was deleted successfully
	 */
	synchronized public static boolean removeLog4JLCF(IProject project){
		plugin.log("removing lcf from project " + project);
		
		try{
			File lcf = project.getLocation().append(LCF).toFile();
			if(lcf.exists()){
				return Files.deleteFile(lcf);
			}else{
				plugin.log("lcf did not exist");
			}
			return true;
		}catch(Exception e){
			plugin.log("Failed to delete lcf for project", e);
			return false;
		}
	}
	
	/**
	 * create anyDynamic File
	 * creates a custom pointcut file for use in the compile
	 * called when the checker is enabled and/or when any options are changed on the properties
	 * page for a project.
	 * @return returns true if the file was created successfully and false otherwise
	 */
	synchronized public static void createCustomPointcutFile(IProject thisProject) throws CoreException{
		plugin.log("creating custom pointcut file for " + thisProject);
		
		try{
			// initialize the new file
			File anyDFile = CheckerProperties.getFormWorkingDir(thisProject).append( 
						   	CHECKER_PACKAGE + File.separator + 
						   	ANYDYNAMIC_FILE).toFile();
			Files.createNewFile(anyDFile);
			
			writeCustomPointcutFile(thisProject, anyDFile);
			
		}catch(Exception e){
			String message = "Error creating custom pointcut file: ";
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
		}
	}
	
	/**
	 * helper function that writes the custom prop file based on the 
	 * choices for this project
	 */
	private static void writeCustomPointcutFile(IProject myProject, File pcFile) throws CoreException {
		try{
			BufferedWriter fileOut = new BufferedWriter(new FileWriter(pcFile));
			
			// top
			fileOut.write(ANYD_TOP);
			fileOut.newLine();
			// toCheck
			fileOut.write(ANYD_POINTCUT + ANYD_TOCHECK);
			fileOut.newLine();
			//toAvoid
			fileOut.write(ANYD_POINTCUT + ANYD_TOAVOID + ":" + 
				CheckerProperties.retrieveSettingString(myProject, 
							CheckerProperties.TO_AVOID_PROPERTY) + ";");
			fileOut.newLine();
			// stable
			String custStable = CheckerProperties.retrieveSettingString(myProject, 
							CheckerProperties.CUST_STABLE_PROPERTY);
			if(!custStable.trim().equalsIgnoreCase("")){
				custStable = "|| (" + custStable + ")";
			}
			fileOut.write(ANYD_POINTCUT + ANYD_STABLE + ":" + 
				CheckerProperties.retrieveSettingString(myProject, 
							CheckerProperties.WHAT_STABLE_PROPERTY) + " " +
				custStable + ";");
							
			fileOut.newLine();
			//global preferred
			String custGlobal = CheckerProperties.retrieveSettingString(myProject,
							CheckerProperties.CUST_GLOBAL_PROPERTY);
			if(!custGlobal.trim().equalsIgnoreCase("")){
				custGlobal = "|| (" + custGlobal + ")";
			}
			fileOut.write(ANYD_POINTCUT + ANYD_GLOBAL + ":" + 
				CheckerProperties.retrieveSettingString(myProject,
							CheckerProperties.WHAT_GLOBAL_PROPERTY) + " " +
				custGlobal + ";");
			fileOut.newLine();

			fileOut.write("}");
			fileOut.close();
		}catch(Exception e){
			String message = "Error writing custom pointcut file: ";
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
		}
	}
	
	/**
	 * shows all of the views related to the LoDChecker in the current perspective
	 * when a checker is enabled
	 * 
	 * @todo when a project is opened that is checker enabled, show the views
	 */
	synchronized public static void showViews() throws CoreException{
		plugin.log("showing views");
		
		try{
			LoDCheckerViolationsView.showView();
		}catch(Exception e){
			String message = "Error showing views: ";
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
		}
	}
	
	/**
	 * hides the views related to the LoDChecker.
	 * currently this method is not used. 
	 * it was originally intended to be used when the checker was disabled, but
	 * there might be a second project open that also is Checker enabled
	 * so it would be undesireable to close the views without first verifying that
	 * no open project is checker enabled
	 * 
	 * @todo verify that there are no other open projects with the checker enabled
	 * when calling this function
	 */
	synchronized public static void hideViews() {
		plugin.log("hiding views");
		
		LoDCheckerViolationsView.hideView();
	}

	/**
	 *  Adds the LoDCheckerNature and AspectJNature to this project
	 * 
	 * natures will not be added if they already exist for this project.
	 */
	synchronized public static void addRequiredNatures(IResource thisResource) throws CoreException {
		boolean addedAspectJ = false;
		
		try{	
			// check to see if contains AspectJ Nature so if adding LoDChecker
			//  nature fails, we can remove AspectJ Nature
			if(!(thisResource.getProject().hasNature(AJ_NATURE))){	
				// add AspectJ nature - add first so AspectJ builder is associated with the project
				// and then our builder can be placed in front of the AspectJ Builder
				Natures.addNature(thisResource, AJ_NATURE);
				addedAspectJ = true;
			}
						
			// add LoDChecker nature
			Natures.addNature(thisResource, LoDCheckerPlugin.ID_NATURE);
			
		}catch(CoreException e){
			String message = "Failed to add AspectJ and LoDChecker Natures";
			plugin.log(message, e);
			e = new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
					
			try{
				// if we added AspectJ Nature and failed to add LoDChecker Nature
				// then remove AspectJ Nature
				if(addedAspectJ){
					Natures.removeNature(thisResource, LoDCheckerPlugin.ID_NATURE);
				}
			}catch(CoreException e2){
				message = "Failed to remove LoDChecker Nature ";
				plugin.log(message, e2);
				e = new CoreException(
					new Status(
						Status.ERROR,
						LoDCheckerPlugin.PLUGIN_ID,
						Status.OK,
						message,
						e2));
			}
			
			throw e;
		}
	}
	/**
	 *  Removes the LoDCheckerNature and AspectJNature to this project.
	 * 
	 * If the project does not have either nature an error is NOT thrown.
	 */
	synchronized public static void removeRequiredNatures(IResource thisResource, boolean promptForAJ) throws CoreException {
		try{		      	
			// Remove LoDCheckerNature
			Natures.removeNature(thisResource, LoDCheckerPlugin.ID_NATURE);				
			
		    // Remove AspectJ Nature
			if(thisResource.getProject().hasNature(AJ_NATURE)){
				boolean remove = true;
				if(promptForAJ){
					remove = 
						Messages.confirm("Confirm AspectJ Nature Removal", 
							"Would you like to disassociate AspectJ and this Project?\n\n" +
							"Press 'OK' to remove the AspectJ Nature from this project.\n" + 
							"Press 'Cancel' to keep maintain the AspectJ Nature of this project.");
				}
				if(remove){
		    	Natures.removeNature(thisResource, AJ_NATURE);
				}
			}
			
		}catch(CoreException e){
			String message = "Failed to remove AspectJ and LoDChecker Natures";
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
		}
	}
	
	/**
	 *  adds any libraries required by the LoDChecker to the classpath
	 * 
	 * The required libraries exist in the checker plugin installation directory's
	 * lib directory.
	 */
	synchronized public static void addRequiredLibs(IProject project) throws CoreException {
			Classpath.addToClasspath(project, 
				(IClasspathEntry[])getRequiredLibraries().toArray(new IClasspathEntry[0]));			
	}
	
	/**
	 *  removes any added required libraries from the LoDChecker classpath
	 * 
	 * The required libraries exist in the checker plugin installation directory's
	 * lib directory.
	 */
	synchronized public static void removeRequiredLibs(IProject project) throws CoreException{
			Classpath.removeFromClasspath(project, 
				(IClasspathEntry[])getRequiredLibraries().toArray(new IClasspathEntry[0]));			
	}
	
	/**
	 * @return returns a Vector of the Classpath entries for all of the libraries 
	 * required by the LoDChecker plug-in.
	 * 
	 * The required libraries exist in the checker plugin installation directory's
	 * lib directory.
	 */
	private static Vector getRequiredLibraries(){
		Vector requiredLibs = new Vector();
		
		// load all files in the lib folder of the plugin filtered by the LibFileFilter
		File libFolder = new File(
			LoDCheckerPlugin.getAbsolutePath() +
			LoDCheckerPlugin.LIB_FILES_DIR + File.separator);
		if(!libFolder.isDirectory()){ return new Vector();}
		
		Vector filters = new Vector();
		filters.add(".jar");
		File[] libFiles = libFolder.listFiles(new LoDFileFilter(filters));
		for(int i = 0; i < libFiles.length; i++){
			requiredLibs.add(JavaCore.newLibraryEntry(
				new Path(libFiles[i].getAbsolutePath()), null, null));
		}
				
		return requiredLibs;
	}	
}



