package edu.neu.ccs.com3205.lodchecker;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;

import java.io.File;

/**
 * CheckerProperties.java
 * Common static interface for retrieving and setting the LoDChecker related
 * properties of a resource.
 * 
 * @author Paul Freeman
 */
public class CheckerProperties {

  private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();

	// string constants representing properties
	public static final QualifiedName ENABLE_PROPERTY = new QualifiedName("", "enabled");
	public static final QualifiedName FORM_PROPERTY = new QualifiedName("", "form");
	public static final QualifiedName TO_AVOID_PROPERTY = new QualifiedName("", "toAvoid");
	public static final QualifiedName WHAT_STABLE_PROPERTY = new QualifiedName("", "whatStable");
	public static final QualifiedName CUST_STABLE_PROPERTY = new QualifiedName("", "custStable");
	public static final QualifiedName WHAT_GLOBAL_PROPERTY = new QualifiedName("", "whatGlobal");
	public static final QualifiedName CUST_GLOBAL_PROPERTY = new QualifiedName("", "custGlobal");
	
	// string constants used in program and as values for properties
	public static final String OBJECT_FORM = "object";
	public static final String CLASS_FORM = "class";
	
	/**
	 * Preserves a boolean value for a named field on the project.
	 * Actually converts the boolean to a string and delegates to the 
	 * other implementation of preserveSetting() that takes a string as the
	 * value to store.
	 */
	synchronized public static void preserveSetting(IResource thisResource, QualifiedName key, boolean flag)
		throws CoreException {
		preserveSetting(thisResource, key, new Boolean(flag).toString());
	}

	/**
	 * Preserve a key/value pair as a persistent property against the 
	 * project.
	 */
	synchronized public static void preserveSetting(IResource thisResource, QualifiedName key, String value)
		throws CoreException {
		thisResource.setPersistentProperty(key, value);
	}

	/** 
	 * Retrieve a persistent property value and return it.  If the key is
	 * not found, this will return null.
	 */
	synchronized public static String retrieveSettingString(IResource thisResource, QualifiedName key) {
		String value = null;
		try {
			value = thisResource.getPersistentProperty(key);
		} catch (CoreException e) {
		}
		return value;
	}

	/** 
	 * Retrieve a persistent property value and return it.  If the key is
	 * not found, this will return the provided defaultVal.
	 */
	synchronized public static String retrieveSettingString(IResource thisResource, QualifiedName key, String defaultVal) {
		String value = retrieveSettingString(thisResource, key);
		if(value != null) {
			return value;
		}else{
			return defaultVal;
		}
	}

	/**
	 * Retrieve a persistent property value, convert it to a boolean and return it.
	 * If the key is not found, this will return null.
	 */
	synchronized public static Boolean retrieveSettingBoolean(IResource thisResource, QualifiedName key) {
		Boolean value = null;
		try {
			value = new Boolean(thisResource.getPersistentProperty(key));
		} catch (CoreException e) {
		}
		return value;
	}

	/**
	 * Retrieve a persistent property value, convert it to a boolean and return it.
	 * If the key is not found, this will return the provided defaultVal.
	 */
	synchronized public static boolean retrieveSettingBoolean(IResource thisResource, QualifiedName key, boolean defaultVal) {
		Boolean value = retrieveSettingBoolean(thisResource, key);
		if(value != null) {
			return value.booleanValue();	
		}else{
			return defaultVal;
		}
	}
	
	/**
	 * @return Returns the working directory of the source files for
	 * the Checker with respect to this project.  
	 */
	synchronized public static IPath getWorkingSourceDir(IProject thisProject){
		IPath srcDir = plugin.getWorkingLocation(thisProject);
		srcDir = srcDir.append(Checker.CHECKER_SRC_DIR);
		
		return srcDir;
	}
	
	/**
	 * @return Returns the working directory for the form of the Checker
	 * being used with this project.  Returns null if there is no form
	 * property set for this project.
	 */
	synchronized public static IPath getFormWorkingDir(IProject thisProject){
		IPath formDir = getWorkingSourceDir(thisProject);
		
		String form = CheckerProperties.retrieveSettingString(
			thisProject,
			CheckerProperties.FORM_PROPERTY);
		if(form != null){
			if(form.equalsIgnoreCase(CheckerProperties.CLASS_FORM)){
			//class form
				formDir = formDir.append(Checker.CLASS_FORM_DIR);
			}else if(form.equalsIgnoreCase(CheckerProperties.OBJECT_FORM)){
			//object form
				formDir = formDir.append(Checker.OBJECT_FORM_DIR);
			}
			
			return formDir;
		}
		return null;
	}
}
