package edu.neu.ccs.com3205.lodchecker;

import org.eclipse.ui.*;
import org.eclipse.jface.viewers.*;
import org.eclipse.ui.plugin.*;
import org.eclipse.core.runtime.*;
import org.eclipse.core.internal.resources.PlatformURLResourceConnection;
import org.eclipse.core.resources.*;
import org.eclipse.jdt.core.*;
import org.eclipse.debug.core.*;

import org.aspectj.ajde.Ajde;
import org.eclipse.ajdt.ui.AspectJPlugin;

import edu.neu.ccs.com3205.lodchecker.utils.*;
import edu.neu.ccs.com3205.lodchecker.internal.*;
import edu.neu.ccs.com3205.lodchecker.ui.*;

import java.util.*;
import java.io.*;

/**
 * LoDCheckerPlugin.java
 * The main plugin class to be used in the desktop.
 * 
 * This class contains multiple static methods that relate to the retreival of
 * usefull checker related information.
 * 
 * @author Paul Freeman
 */
public class LoDCheckerPlugin extends AbstractUIPlugin  {
			
	
	private static LoDCheckerPlugin plugin; 	//The shared instance.
	private ResourceBundle resourceBundle;			//Resource bundle.
	private PrintStream logPrintStream;				// prinstream for logging
	
	// plug-in specific constants
	public static final String PLUGIN_ID = "edu.neu.ccs.com3205.LoDChecker";
	public static final String ID_NATURE = PLUGIN_ID + ".LoDCheckerNature";
	public static final String ID_BUILDER = PLUGIN_ID + ".LoDCheckerBuilder";
	public static final String ID_VIOLATION_VIEW = LoDCheckerViolationsView.VIEW_ID;
	public static final String LIB_FILES_DIR = "lib";
	public static final String ICONS_DIR = "icons";
	
	/**
	 * The constructor.
	 */
	public LoDCheckerPlugin(IPluginDescriptor descriptor) {
		super(descriptor);
		plugin = this;
		
		// use for debugging
		plugin.enableLogging(new File(LoDCheckerPlugin.getAbsolutePath() + "LoDChecker.log"));
		plugin.log("initialized LoDChecker plugin " + new Date());
		
		try {
			resourceBundle = ResourceBundle.getBundle("edu.neu.ccs.com3205.LoDChecker.LoDCheckerPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
		
		LoDCheckerPlugin.startAspectJPlugin();
		
		// add the event listeners required by this plugin to the workbench
		IWorkbenchWindow theWindow = plugin.getWorkbench().getActiveWorkbenchWindow();
		theWindow.getSelectionService().addSelectionListener(new LoDSelectionListener());
  	DebugPlugin.getDefault().getLaunchManager().addLaunchListener(new LoDLaunchListener());
	}
	
//	/**
//	 * Will be called in a separed thread after the workbench initializes.
//	 * 
//	 * Registers workbench listeners required by this plugin.
//	 */
//	public void earlyStartup(){
//				
//		// there is only one workbench window at earlyStartup, however it is not
//		// active so we cannot use getActiveWorkbenchWindow to retrieve it :(
//		// (that returns null).  Instead we access it through the workbench window array
//		IWorkbenchWindow theWindow = plugin.getWorkbench().getWorkbenchWindows()[0];
//		
////		LoDCheckerPlugin.startAspectJPlugin();
//			
//		// register a window listener for this plug-in
//		plugin.getWorkbench().addWindowListener(new LoDWindowListener());
//		
//		// register a selection change listener for this plug-in
////		theWindow.addPageListener(new LoDPageListener());
//		theWindow.getSelectionService().addSelectionListener(new LoDSelectionListener());
//		theWindow.addPerspectiveListener(new LoDPerspectiveListener());
////		theWindow.getPartService().addPartListener(new LoDPartListener());
//	}
	
	/**
	 * @return Returns the shared instance.
	 */
	public static LoDCheckerPlugin getDefault() {
		return plugin;
	}

	/**
	 * static method that will start the LoDCheckerPlugin
	 */
	public static void startPlugin(){
		// start the LoDCheckerPlugin
		try{
			Platform.getPluginRegistry().getPluginDescriptor(LoDCheckerPlugin.PLUGIN_ID).getPlugin();
			plugin.log("started LoDChecker plugin " + new Date());
		}catch(Exception e){
			plugin.log("Failed to activate LoDCheckerPlugin!", e);
		}
	}
	
	/**
	 * static method that will start the required AspectJPlugin
	 * This method is automatically called when the LoDCheckerPlugin is started
	 */
	public static void startAspectJPlugin(){
		// start the AspectJPlugin
		try{
			Platform.getPluginRegistry().getPluginDescriptor(AspectJPlugin.PLUGIN_ID).getPlugin();
			plugin.log("started AspectJ plugin " + new Date());
		}catch(Exception e){
			plugin.log("Failed to activate AspectJPlugin!", e);
		}
	}
	
	/**
	 * @return Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * @return Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	synchronized public static String getResourceString(String key) {
		ResourceBundle bundle= LoDCheckerPlugin.getDefault().getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * @return Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
	
	/**
	 * @return Returns the current AspectJ BuildManager if one is available,
	 * else returns null. 
	 */
	public static org.aspectj.ajde.BuildManager getCurrBuildManager(){
		try{
		// will throw error if plugin is not active
			return Ajde.getDefault().getBuildManager();
		}catch(RuntimeException e){
			return null;
		}
	}
	
	/**
	 * @return Returns the absolute path to this plugin's directory.
	 * Returns null if the directory cannot be determined.
	 */
	public static String getAbsolutePath(){
		try{
			return 
				Platform.resolve(plugin.getDescriptor().getInstallURL()).getFile();
		}catch(java.io.IOException e){
			plugin.log("Failed to resolve plugin URL for absolute path", e);
		}
		return null;
	}

	/**
	 * @return Returns the location of the directory available for use by this plugin
	 * specific to the given project.
	 */
	public IPath getWorkingLocation(IProject project){
		return project.getPluginWorkingLocation(plugin.getDescriptor());
	}
	
	/**
	 * @return Returns the LoDChecker buildFile for this project.
	 */
	public IPath getBuildConfigurationFile(IProject project){
//		return getWorkingLocation(project).append(Checker.LST_FILE);
		return project.getLocation().append(Checker.LST_FILE);
	}
	
	/**
	 * @return Returns an IPath to the temporary violations file for this project
	 */
	public static IPath getTempViolationsFile(IProject project) {
		return plugin.getWorkingLocation(project).append(Checker.TEMP_VFILE_NAME);	
	}
	
	/**
	 * @return Returns an IPath to the persistent violations file, i.e. the violations
	 * file that contains violations from previously executed session(s)
	 */
	public static IPath getPersistentViolationsFile(IProject project){
		return project.getLocation().append(Checker.PERSIST_VFILE_NAME);
	}
	
/////////////////////////
// logging for debugging
/////////////////////////
	/**
	 * Enables logging for this plugin.  
	 */
	public void enableLogging(File logFile){
		if(!logFile.exists()){
			Files.createNewFile(logFile);
		}
		try{
			logPrintStream = new PrintStream(new FileOutputStream(logFile));
		}catch(FileNotFoundException e){
			
		}
	}
	
	/**
	 * Disables the logging for this plugin
	 */
	public void disableLogging(){
		logPrintStream = null;
	}
	
	/**
	 * Logs a string to the log file
	 */
	public void log(String message){
		if(logPrintStream != null){
			logPrintStream.println("<LoD> " + message);
			logPrintStream.flush();
		}
	}

	/**
	 * Logs a string and an exception to the log file
	 */
	public void log(String message, Exception e){
		if(logPrintStream != null){
			this.log(message);
			this.log("ERROR:");
			if(e instanceof CoreException){
				this.log("Plugin: " + ((CoreException)e).getStatus().getPlugin());
				this.log("Message: " + ((CoreException)e).getStatus().getMessage());
				this.log("Severity: " + ((CoreException)e).getStatus().getSeverity());
				this.log("Exception: " + ((CoreException)e).getStatus().getException());
			}
			e.printStackTrace(logPrintStream);
		}
	}
}
