package edu.neu.ccs.com3205.lodchecker;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.ajdt.ui.AspectJPlugin;

import java.io.*;
import java.util.*;

import edu.neu.ccs.com3205.lodchecker.utils.*;
import edu.neu.ccs.com3205.lodchecker.internal.*;

/**
 * LoDCheckerProjectNature.java
 * This class adds and removes the appropriate builder for LoDChecker
 * enabled projects.  It also adds and removes all of the files required by
 * the checker to and from the plugin's working directory for this project.
 * 
 * @author Paul Freeman
 */
public class LoDCheckerProjectNature implements IProjectNature {

  private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();

	private IProject project;

	/**
	 * Driven when this project nature is 'given' to a project,
	 * it adds the appropriate builder to the project build specification.
	 * It assumes the project already contains the AspectJ Builder as that
	 * nature should be added before our nature.
	 * 
	 * All of the files required by our checker are also added to the
	 * plugin's working directory for this project at the time our nature
	 * is enabled
	 */
	public void configure() throws CoreException {
		plugin.log("configuring LoD nature");
		
		try{
//			// reset the AspectJ pluging to use the default build file
//			Builder.resetBuildConfigurationFile(project);
	
			// add our builder to the project's list of builders
			addBuilder(LoDCheckerPlugin.ID_BUILDER);
			
			// add the java files required by our checker
			copyCheckerFiles(project);
		}catch(CoreException e){
			plugin.log("Failed to configure LoDChecker Nature!", e);
			throw e;
		}
	}

	/**
	 * Driven when this project nature is 'removed' from a project,
	 * it removes the appropriate builder from the project build specification.
	 * 
	 * All of the files required by our checker are also removed from the
	 * plugin's working directory for this project at the time our nature
	 * is disabled
	 */
	public void deconfigure() throws CoreException {
		plugin.log("deconfiguring LoD nature");
		
		try{
			// reset the AspectJ pluging so it does not use our build file
			Builder.resetBuildConfigurationFile(project);
	
			// remove our builder from the list of builders for this project
			removeBuilder(LoDCheckerPlugin.ID_BUILDER);
			
			// delete all the java files that our checker uses
			deleteCheckerFiles(project);
		}catch(CoreException e){
			plugin.log("Failed to deconfigure LoDChecker Nature!", e);
			throw e;
		}
	}

	/**
	 * @see IProjectNature#getProject
	 */
	public IProject getProject() {
		return project;
	}

	/**
	 * @see IProjectNature#setProject
	 */
	public void setProject(IProject value) {
		project = value;
	}

	/**
	 * Does all the work of adding our builder to the project
	 */
	private void addBuilder(String builderId) throws CoreException {
   	try{
			plugin.log("adding our builder to " + getProject());
		
			if(!hasBuilder(project, builderId)){
				// create our builder command
				IProjectDescription projectDescription = project.getDescription();
				ICommand command = projectDescription.newCommand();
				command.setBuilderName(builderId);
		
				// Insert our build command to the front of the list
				// we are assuming here that the AspectJ build command
				// is in the list since the AspectJ Project Nature 
				// was added before the LoDChecker Project Nature
				ICommand[] buildCommands = projectDescription.getBuildSpec();
		
				ICommand[] newBuildCommands;
				newBuildCommands = insert(buildCommands, command);
				
				projectDescription.setBuildSpec(newBuildCommands);
				project.setDescription(projectDescription, new NullProgressMonitor());
			}else{
				plugin.log(getProject() + " already has " + builderId);
			}				
		}catch(CoreException e){
			plugin.log("Failed to add " + builderId, e);
			throw e;
		}
	}

	/**
	 * Does all the work of removing our builder from the project
	 */
	private void removeBuilder(String builderId) throws CoreException {
		try{
			plugin.log("removing our builder from " + getProject());
			
			if(hasBuilder(project, builderId)){
				// remove build command from the list of build commands
				IProjectDescription description = project.getDescription();
				ICommand[] buildCommands = description.getBuildSpec();
				
				ICommand[] newBuildCommands;
				newBuildCommands = remove(buildCommands, builderId);
				
				description.setBuildSpec(newBuildCommands);
				project.setDescription(description, new NullProgressMonitor());
			}else{
				plugin.log(project + " did not have " + builderId);
			}				
		}catch(CoreException e){
			plugin.log("Failed to remove " + builderId, e);
			throw e;
		}
	}

	/**
	 * @return returns true if project has a builder with the builderId and false otherwise
	 */
	private boolean hasBuilder(IProject project, String builderId) throws CoreException{
		IProjectDescription description = project.getDescription();
		ICommand[] buildCommands = description.getBuildSpec();
		boolean found = false;

   	for (int i = 0; i < buildCommands.length; ++i) {
    	if (buildCommands[i].getBuilderName().equals(builderId)) {
      	found = true;
      	break;
    	}
   	}
   	return found;
	}

	/**
	 * Insert a new build command at the front of an existing list
	 */
	private ICommand[] insert( ICommand[] sourceCommands, ICommand command ) {
		ICommand[] newCommands = new ICommand[ sourceCommands.length + 1 ];
		newCommands[0] = command;
		for (int i = 0; i < sourceCommands.length; i++ ) {
			newCommands[i+1] = sourceCommands[i];
		}		
		return newCommands;		
	}
	
	/**
	 * Removes all instances of a build command from a list of commands
	 */
	private ICommand[] remove( ICommand[] sourceCommands, String builderId ) {
		Vector newCommands = new Vector();
		for (int i = 0; i < sourceCommands.length; i++ ) {
			if ( !sourceCommands[i].getBuilderName( ).equals( builderId ) ) {
				newCommands.add(sourceCommands[i]);
			}
		}		
		return (ICommand[])newCommands.toArray(new ICommand[0]);		
	}
	
	/**
	 * Copies all of the required checker files for this project
	 * to the project's working location for the LoDCheckerPlugin
	 * effectively hiding the LoDChecker files from the user
	 */
	synchronized private static void copyCheckerFiles(IProject thisProject) throws CoreException {
		try{
	//		Messages.confirm("copying project files to " + thisProject);
						
			// get the from and to path roots for copying the files
			IPath toPath = CheckerProperties.getWorkingSourceDir(thisProject);
			String fromPath = LoDCheckerPlugin.getAbsolutePath() + Checker.CHECKER_SRC_DIR + File.separator;
			
			// copy the files
			Files.copyAll(new File(fromPath), toPath.toFile(), true);		
		}catch(Exception e){
			String message = "Error coping checker files for " + thisProject;
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
		}		
	}
	
	/**
	 * Deletes all of the required checker files from the project's working location
	 * for the LoDCheckerPlugin.
	 */
	synchronized private static boolean deleteCheckerFiles(IProject thisProject){
		try{
	//		Messages.confirm("deleting files from the project " + thisProject);
			
			// get the directory where checker files were stored
			IPath checkerPath = plugin.getWorkingLocation(thisProject);
			IPath buildFile = plugin.getBuildConfigurationFile(thisProject);
			
			// delete the files
			return Files.deleteAll(checkerPath.toFile()) && Files.deleteFile(buildFile.toFile());		
		}catch(Exception e){
			plugin.log("Error deleting checker files for " + thisProject, e);
			return false;
		}		
	}	
}