package edu.neu.ccs.com3205.lodchecker.internal;

import java.util.*;
import java.io.*;

import org.eclipse.ajdt.internal.ui.ajde.*;
import org.eclipse.ajdt.ui.AspectJPlugin;
import org.aspectj.ajde.BuildListener;
import org.aspectj.ajde.BuildManager;
import org.eclipse.ajdt.internal.ui.AspectJPreferences;

import org.eclipse.jface.preference.*;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;

import edu.neu.ccs.com3205.lodchecker.*;
import edu.neu.ccs.com3205.lodchecker.utils.*;

/**
 * Builder.java
 * This builder writes a custom LoDCheckerBuild.lst file and sets the AspectJ
 * Plugin to use this file, just prior to a build with AspectJ.
 * This is possible if and only if our builder is placed ahead of the AspectJ
 * Builder in the list of build commands for this project.
 * 
 * @author Paul Freeman
 */

public class Builder extends IncrementalProjectBuilder implements BuildListener {

  private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();

	/**
	 * The name of the default build config file for an AspectJ project
	 * This is read from the AspectJ plugin
	 */
	public static String AJ_DEFAULT_CONFIG_FILE = org.eclipse.ajdt.internal.builder.Builder.DEFAULT_CONFIG_FILE;


	// stores the original build file used by AspectJ to be reset when build succeeds
	private static HashMap originalBuildFiles = new HashMap();
	
	/**
	 * Constructor for LoDChecker Builder
	 */
	public Builder() {	
		plugin.log("initializing builder");
	}

	/**
	 * @see IncrementalProjectBuilder#build(int, Map, IProgressMonitor)
	 * kind is one of: FULL_BUILD, INCREMENTAL_BUILD or AUTO_BUILD
	 */
	protected IProject[] build(
		int kind,
		Map args,
		IProgressMonitor progressMonitor) 
		throws CoreException {			
			plugin.log("started " + getBuildKindString(kind) + " at " + new Date());
			
			// if AspectJ is suppressing an autobuild and this is autobuild then suppress too
			IPreferenceStore store = AspectJPlugin.getDefault().getPreferenceStore();
			if(store.getBoolean(AspectJPreferences.AUTOBUILD_SUPPRESSED) &&
				 kind == IncrementalProjectBuilder.AUTO_BUILD){
				 	plugin.log("autobuild is suppressed");
				 	return null;
			}

			// get current project
			IProject thisProject = getProject();
			
			// get the current build configuration file
			String buildFile = AspectJPlugin.getBuildConfigurationFile(thisProject);
			
			// proceed only if current build file is not our custom build file
			String projectLoDBuildFile = plugin.getBuildConfigurationFile(getProject()).makeAbsolute().toOSString();
			if(!buildFile.equalsIgnoreCase(projectLoDBuildFile)){
				plugin.log("Current AspectJ Build File: " + buildFile);
				plugin.log("Creating New Project Build File: " + projectLoDBuildFile);

				IFile originalBuildFile = thisProject.getFile(new Path(new File(buildFile).getName()));
				originalBuildFiles.put(thisProject, originalBuildFile);
				
				ProjectProperties props = AspectJPlugin.getDefault().getAjdtProjectProperties();
				List projectFiles = props.getProjectSourceFiles( thisProject );
	
				// if using the AspectJ default build file, rewrite it before compile
				if(buildFile.endsWith(AJ_DEFAULT_CONFIG_FILE)){
					writeBuildConfigFile(projectFiles, thisProject, buildFile);
				}
					
				// build a new LoDChecker configuration file
				buildFile = buildNewConfigFile(originalBuildFile.getLocation().toOSString());
				
				// set the AspectJ pluging to use the LoDChecker Build file
				Path buildFilePath = new Path(new File(buildFile).getAbsolutePath());
				IFile newBuildConfigFile = thisProject.getFile(Files.findRelativePath(thisProject.getLocation().makeAbsolute(), buildFilePath));
plugin.log("path from Project to New Build File: " + newBuildConfigFile.getFullPath());
				setBuildConfigurationFile(thisProject, newBuildConfigFile);

				this.activateListener();				
			}else{
				plugin.log("build started with LoDChecker build file");
			}
			
			// return a null list of required resource deltas (yes could just say return null)
			IProject[] requiredResourceDeltasOnNextInvocation = null;
			return requiredResourceDeltasOnNextInvocation;			
	}

	/**
	 * Create a full build configuration file for this project<br>
	 * 
	 * This method was taken almost completely intact from the AspectJ plugin.
	 */
	private void writeBuildConfigFile(List projectFiles, IProject project, String buildFile)
		throws CoreException {
		plugin.log("ReWriting " + buildFile);

		try {
			FileWriter fw = new FileWriter(buildFile);
			BufferedWriter bw = new BufferedWriter(fw);
			for (Iterator it = projectFiles.iterator(); it.hasNext();) {
				File jf = (File) it.next();
				String fileName = jf.toString();
				if (fileName.endsWith(".java") || fileName.endsWith(".aj")) {
					bw.write(fileName);
					bw.newLine();
				}
			}
			bw.flush();
			fw.flush();
			fw.close();
			bw.close();			
		} catch (Exception e) {
			plugin.log("Failed to create default build configuration file.", e);
			Status status =
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					"Failed to create default build configuration file.",
					e);
			throw new CoreException(status);
		}
	}

	/**
	 * creates a new configuration file to use based on the original
	 * configuration file and returns the location of the new file
	 * @return returns null if the new file cannot be created
	 */
	private String buildNewConfigFile(String buildConfigFile){
		try{
			// initialize files
			File buildFile = new File(buildConfigFile);
			File newConfigFile = plugin.getBuildConfigurationFile(getProject()).toFile();

			if(!buildFile.getAbsolutePath().equalsIgnoreCase(newConfigFile.getAbsolutePath())){
				if(!Files.createNewFile(newConfigFile)){
					return null;
				}
		
				// combine the two files into the new configuration file
				BufferedWriter fileOut = new BufferedWriter(new FileWriter(newConfigFile));
				BufferedReader fileIn;
				String nextLine;
				
				// add the current buildFile values
				HashMap buildFileLines = new HashMap();
				fileIn = new BufferedReader(new FileReader(buildFile));
				while((nextLine = fileIn.readLine()) != null){
					fileOut.write(nextLine);
					fileOut.newLine();
					buildFileLines.put(nextLine, null);
				}
				fileIn.close();
				
				// add files for form
				Vector filters = new Vector();
				filters.add(".java");
				filters.add(".aj");
				LoDFileFilter filter = new LoDFileFilter(filters);
				File[] formFiles = Files.getAll(CheckerProperties.getFormWorkingDir(getProject()).toFile(), filter);
				for(int f = 0; f < formFiles.length; f++){
					nextLine = formFiles[f].getAbsoluteFile().toString();
					if(!buildFileLines.containsKey(nextLine)){
						fileOut.write(nextLine);
						fileOut.newLine();
					}
				}
				
				// close and return the new file
				fileOut.close();
			}
			return newConfigFile.getAbsoluteFile().toString();
		}catch(Exception e){
			plugin.log("Error combining list files.", e);
			return null;		
		}
	}


/////////
// AspectJ Build Manager Listener interface starts here
/////////
    /**
     * Called when a new build is started.
     * 
     * @param   buildConfigFile configuration file used for the last compil
     */
    public void compileStarted(String buildConfigFile){
  		// do nothing - required by BuildListener interface
 			plugin.log("AspectJ compile started with " + buildConfigFile);
    }

    /**
     * Called when a build completes.
     * 
     * @param   buildConfigFile configuration file used for the last compile
     * @param   buildTime   	 compilation time in miliseconds
     * @param   succeeded   	 true if build succeeded without errors
     */
    public void compileFinished(String buildConfigFile, int buildTime, boolean succeeded, boolean warnings){
 			plugin.log("AspectJ compile finished using " + buildConfigFile);

  		// reset the build file for this project to the original build file
  		compileTerminated();
   }
    
    /**
     * Called when a build is aborted before completion.
     * 
     * @param	buildConfigFile	configuration used for the last compile
     * @param	message			message explaining reason for abort
     */
    public void compileAborted(String buildConfigFile, String message){			
 			plugin.log("AspectJ compile aborted using " + buildConfigFile);
  		plugin.log("AspectJ compiler message: " + message);

  		// reset the build file for this project to the original build file
  		compileTerminated();
    }
//// END aspectJ build listener interface

	/**
	 * helper function that performs functions when compile finishes or aborts
	 */
	private void compileTerminated(){
  	resetBuildConfigurationFile(getProject());
		this.deactivateListener();
	}
	
	/**
	 * public function to set the build configuration file
	 */
	synchronized public static void setBuildConfigurationFile(IProject project, IFile file){
		// supress the automatic rebuild when list file is changed
		IPreferenceStore store = AspectJPlugin.getDefault().getPreferenceStore();
		boolean oldVal = store.getBoolean(AspectJPreferences.AUTOBUILD_SUPPRESSED);
		store.setValue(AspectJPreferences.AUTOBUILD_SUPPRESSED, true);
		
		// change list file
  	AspectJPlugin.getDefault().setBuildConfigurationFile(project, file);	
 		plugin.log("Set build file to " + file.getFullPath().toOSString());

		// reset the automatic rebuild option after list file is changed
		store.setValue(AspectJPreferences.AUTOBUILD_SUPPRESSED, oldVal);
	}
	
	/**
	 * public function to reset the build configuration file in case build options change
	 */
	synchronized public static void resetBuildConfigurationFile(IProject project){
		plugin.log("Resetting build file.");
		
		IFile originalBuildFile = (IFile)originalBuildFiles.get(project);
		if(originalBuildFile == null){
  		setBuildConfigurationFile(project, project.getFile(AJ_DEFAULT_CONFIG_FILE));	
		}else{
  		setBuildConfigurationFile(project, (IFile)originalBuildFiles.get(project));	
		}
  }
	
	/**
	 * Activates this listener.
	 */
	protected boolean activateListener(){
		BuildManager bm = LoDCheckerPlugin.getCurrBuildManager();
		if(bm != null){
			bm.addListener(this);
  		plugin.log("Added Builder as AspectJ Build Listener");
			return true;
		}
  	plugin.log("Failed to add Builder as AspectJ Build Listener");
		return false;
	}
	
	/**
	 * Deactivates this listener.
	 */
	protected boolean deactivateListener(){
		BuildManager bm = LoDCheckerPlugin.getCurrBuildManager();
		if(bm != null){
			bm.removeListener(this);
  		plugin.log("Deactivated Builder as AspectJ Build Listener");
			return true;
		}
  	plugin.log("Failed to deactivate Builder as AspectJ Build Listener");
		return false;
	}

	/**
	 * helper function that returns the kind as a string.<br>
	 * used in debugging.
	 */
	private String getBuildKindString(int kind){
		switch (kind){
		case IncrementalProjectBuilder.AUTO_BUILD:
			return "AUTO_BUILD";
		case IncrementalProjectBuilder.FULL_BUILD:
			return "FULL_BUILD";
		case IncrementalProjectBuilder.INCREMENTAL_BUILD:
			return "INCREMENTAL_BUILD";
		default:
			return "UNKOWN_BUILD TYPE";
		}
	}
}
