package edu.neu.ccs.com3205.lodchecker.internal;

import org.eclipse.core.resources.*;
import org.eclipse.debug.core.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.debug.core.model.*;
import org.eclipse.core.runtime.*;

import edu.neu.ccs.com3205.lodchecker.*;
import edu.neu.ccs.com3205.lodchecker.ui.*;

/**
 * LoDLaunchListener.java
 * This class monitors launches (or 'run' and 'debug' events) of a project
 * being tested by the user.  It is used to add a listener to the Checker log
 * file that is produced by the normal checker code.  As violations are written
 * to this file, they are read by the monitor and rewritten to another log file
 * which contains only the most recent session run.  This second log file is
 * stored in the working location of the project with respect to this plugin, effectively
 * hiding it from the user to make it more difficult for the user to delete.
 * 
 * @author Paul Freeman
 */
public class LoDLaunchListener implements ILaunchListener , IDebugEventSetListener {

	private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();
	
	// the project last selected
	private static IProject currentProject = null;
	
	/**
	 * sets the current project which is the project whose launch is to be monitored.
	 * Called whenever a new project is selected as monitored by the LoDCheckerSelectionListener
	 * 
	 * @see LoDCheckerSelectionListener
	 */
	synchronized public static void setCurrentProject(IProject project){
		currentProject = project;
	}
	
/////////
// ILaunchListener interface starts here
/////////
	
	/**
	 * Notifies this listener that the specified launch has been added. 
	 * 
	 * event is thrown on the same thread as the workbench itself
	 */
	public void launchAdded(ILaunch launch) {
		if(Checker.isEnabled(currentProject)){
			plugin.log("launch added: " + launch);
	
			// verify that the log configuration file exists 
			try{
				Checker.addLog4JLCF(currentProject);
			}catch(CoreException e){
				// do nothing?? - should perhaps terminate the launch here, but will do nothing for now
				plugin.log("failed to copy lcf before launch", e);
			}
			
			// start monitoring debug events for a process creation event
			DebugPlugin.getDefault().addDebugEventListener(this);		
		}
	}
          
	/**
	 * Notifies this listener that the specified launch has changed. 
	 * 
	 * Event is thrown on thread separate from the workbench.
	 * Event is thrown when a process is added to the launch. 
	 * Event is not thrown at launch finish.
	 */
	public void launchChanged(ILaunch launch) {
		plugin.log("launch changed: " + launch);		
		// do nothing
	}
  
  /**
   * Notifies this listener that the specified launch has been removed. 
   * 
   * Event is not thrown when launch terminates.
   */       
	public void launchRemoved(ILaunch launch) {
		plugin.log("launch removed: " + launch);
		// do nothing as this does not correspond to a launch termination event
	}

//////////
// IDebugEventSetListener interface starts here
//////////

	/**
	 * Notifies this listener of the given debug events. 
	 */
	public void handleDebugEvents(DebugEvent[] events) {
//		plugin.log("handling debug event");
		// examine the events for a events from processes
		if(events != null){
			for(int i = 0; i < events.length; i++){
				DebugEvent thisEvent = events[i];
//				plugin.log("event kind = " + thisEvent.getKind());
//				plugin.log("event source = " + thisEvent.getSource());
				Object src = thisEvent.getSource();
				if(src instanceof IProcess){
					
					// if this is a creation event
					if(thisEvent.getKind() == DebugEvent.CREATE){
						plugin.log("process create event received");
						startMonitor(((IProcess)src).getLaunch());
					}
					
					// if this is a termination event
					if(thisEvent.getKind() == DebugEvent.TERMINATE){
						plugin.log("process terminate event received");
						DebugPlugin.getDefault().removeDebugEventListener(this);
					}
					
				}
			}
		}
	}

///////////
// end IDebugEventSetListener interface
///////////

	/**
	 * Starts the violation file monitor which records this session of violations
	 * The file monitor will stop monitoring when the launch it is created under
	 * terminates.
	 */
	private void startMonitor(ILaunch launch){
		if(launch.getProcesses().length >= 1){
			LoDCheckerViolationsView curView = LoDCheckerViolationsView.getActiveView();
			if(curView != null){
				Display currentDisplay = curView.getViewDisplay();
				ViolationFileMonitor monitor = 
					new ViolationFileMonitor(launch, 
																		currentProject, 
																		currentDisplay, 
																		curView);
				
				// start the file monitor on a thread that will execute asynchronously with the
				// current display thread but can still communicate with the current display thread
				currentDisplay.asyncExec(monitor);
			}else{
				ViolationFileMonitor monitor = 
					new ViolationFileMonitor(launch, 
																		currentProject, 
																		null, 
																		null);
				
				// start the file monitor on a thread that will execute asynchronously with this
				// thread.  The monitor will not communicate with the display since view and display
				// are passed in as null
				new Thread(monitor).start();
			}
		}
	}
}