package edu.neu.ccs.com3205.lodchecker.internal;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.debug.core.*;
import org.eclipse.debug.core.model.*;
import org.eclipse.swt.widgets.*;

import java.util.*;
import java.io.*;

import edu.neu.ccs.com3205.lodchecker.*;
import edu.neu.ccs.com3205.lodchecker.ui.*;


/**
 * ViolationFileMonitor.java
 * This class is used to monitor the persistent log file that is written to by
 * the LoDChecker files.  This class poles the persistent violation log file
 * copying any new information which is written to a seperate log file stored
 * in the working location for the currentProject which is specific to this plugin.
 * This design addresses two issues:<br>
 * This location is effectively hidden from the user, making it less likely that
 * the user will delete the file.<br>
 * Also, the Log4J library, which is currently used
 * by the Checker, can maintain the persistent file as a 'rolling' log file with a 
 * maximum size.  Once the max size is reached, the persistent file is erased and 
 * started anew, perhaps erasing some of the violation information for a current
 * session.  By monitoring violations written to this file, instead of using the
 * persistent file, we are able to ensure that all violations for a particular 
 * will be saved for loading into the violations view.
 * 
 * @author Paul Freeman
 */
public class ViolationFileMonitor implements Runnable {

	private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();
		
	// file writing variables
  private BufferedWriter vFileOut = null;		// output stream to the temporary log file
  private BufferedReader vFileIn = null;		// input stream from the Checker log file
  private File vInFile = null;							// the Checker log file
  private long lastLen = 0;									// the last known length of the checker log file
  																						//  we monitor the length as it could be reduced by
  																						//  the Checker logger
	
	private IProject project = null;	// the project containing the file being monitored
	private ILaunch launch = null;		// the launch this monitor was created under
	private Display display = null;
	private LoDCheckerViolationsView violationsView = null;	// the currently visible view
	private boolean inLightMode = false;
	
	/**
	 * constructs a new ViolationFileMonitor to monitor for the duration
	 * the launch variable is running and to monitor the ViolationLogFile in the
	 * working directory of the project
	 * 
	 * @param launch - the launch that started this monitor
	 * @param project - the project who's violation log file this object is monitoring
	 */
	public ViolationFileMonitor(ILaunch launch, IProject project, Display display, LoDCheckerViolationsView violationsView){
		this.launch = launch;
		this.project = project;
		this.display = display;
		this.violationsView = violationsView;
		if(violationsView != null){
			inLightMode = violationsView.getInLightMode();		
		}
	}
	
/////////
// Runnable interface starts here
/////////
	
	/**
	 * starts this monitor object.  this function is intended to be called
	 * by the Thread whose constructor this object is passed to.  Called when
	 * the start() method of the new thread is called.
	 */
	public void run() {
		startRecordingViolations();
		monitorViolationsFile();		
		stopRecordingViolations();
	}
  
/////////
// end Runnable interface
/////////

	// Starts writing all new violations to a temporary violations file.
	// Calling this function erases any previously recorded violations.
	private void startRecordingViolations() {
		plugin.log(this + " - started recording violations");

		if(project != null){
			try{
				// write a new temporary violations file
				File vFile = plugin.getTempViolationsFile(project).toFile();
				vFileOut = new BufferedWriter(new FileWriter(vFile));
				
				// open a stream to the violation file being written to
				vFile = plugin.getPersistentViolationsFile(project).toFile();
				loadVFileIn(vFile);
				
				// initialize the violations display to begin receiving input
				initializeViolationsDisplay();
			}catch(IOException e){
				plugin.log("Failed to start recording violations", e);
			}
		}else{
			plugin.log("Failed to start recording violations - currentProject = null");
		}
	}
	
	// does all of the monitoring of the violations file
	private void monitorViolationsFile(){
		plugin.log(this + " - starting violation log file monitoring");
		
		try{
			String nextLine;
			// monitor until launch is terminated - events will not be captured by this
			// thread while spinning here
			while(((nextLine = vFileIn.readLine()) != null) || !launch.isTerminated()){
				// copy until at the end of the BufferedReader
				do{
					if(nextLine != null){
						writeViolation(nextLine);
						writeToViolationsDisplay(nextLine);
					}
				}while((nextLine = vFileIn.readLine()) != null);
				
				// if the file has been reduced in size reset the input stream
				if(vInFile.length() < lastLen) { resetVFileIn(); }
				lastLen = vInFile.length();		// reset the stored length
				try{
					Thread.sleep(10);
				}catch(InterruptedException e){
					// continue
				}
			}
			plugin.log("launch terminated");
		}catch(IOException e){
			plugin.log("Failed during vFileIn monitoring.", e);
		}
	}

	// Stops writing all new violations to a temporary violations file.
	// Calling this function finalizes the file and makes it available for reading.
	private void stopRecordingViolations() {
		plugin.log(this + " - stopped recording violations");

		try{
			closeVFileOut();
			closeVFileIn();
						
		}catch(IOException e){
			plugin.log("Failed to stop recording violations", e);
		}
		
		// reload the violations from the newly created file
		reloadViolationsDisplay(project);
	}
	
	//writes a violation to the violation file of the current project
	// helper function that stores the violation item
	private void writeViolation(String violation){
		// store the violation in the violation file if currently writing to file
		if(violation == null) return;
		
		try{
			if(vFileOut != null){
				vFileOut.write(violation);
				vFileOut.newLine();
				vFileOut.flush();
			}
		}catch(IOException e){
			plugin.log("Failed to write to violation file: " + violation, e);
		}
	}
	
	// helper function that initializes the log fileIn variables
	private void openVFileIn(File inFile) throws IOException {
		vInFile = inFile;
		if(!vInFile.exists()){ vInFile.createNewFile(); }
		vFileIn = new BufferedReader(new FileReader(vInFile));
		lastLen = 0;
	}
	
	// helper function that initializes the log fileIn variables and 
	// moves the stream to the end of the file
	private void loadVFileIn(File inFile) throws IOException {
		openVFileIn(inFile);
		lastLen = vInFile.length();
		vFileIn.skip(lastLen);	// skip to the end of the file
	}
	
	// helper function that closes the log fileIn variables
	private void closeVFileIn() throws IOException {
		if(vFileIn != null){
			vFileIn.close();
			vFileIn = null;
		}
		vInFile = null;
		lastLen = 0;
	}
	
	// helper function the closes then reopens the log fileIn variables
	private void resetVFileIn() throws IOException {
		File inFile = vInFile;
		closeVFileIn();
		openVFileIn(inFile);
	}
	
	// helper function that closes the log fileOut variables
	private void closeVFileOut() throws IOException {
		if(vFileOut != null){
			vFileOut.flush();
			vFileOut.close();
			vFileOut = null;
		}
	}
	
	/**
	 * initializes the violations display 
	 */
	private void initializeViolationsDisplay(){
		if(violationsView != null){
			plugin.log("Initializing the violations display...");
			violationsView.initializeDisplay();
			plugin.log("Display initialized...");
		}
	}
	
	/**
	 * writes to the violations display on an Asynchronous thread to this thread.
	 * i.e. this runnable will be executed without locking this thread.
	 * Uses the class VDisplayWriter to do so.
	 * @see VDisplayWriter
	 */
	private void writeToViolationsDisplay(String message){
		if(violationsView != null && !inLightMode){
			plugin.log("Writing message to violations display...");
			violationsView.addToDisplay(message, false);
			plugin.log("message written...");
		}
	}
	
	/**
	 * reloads the violations display 
	 */
	private void reloadViolationsDisplay(IProject project){
		if(violationsView != null){
			plugin.log("Reloading violations display...");
			violationsView.loadViolations(project, true);
			plugin.log("display reloaded...");
		}
	}
}
