package edu.neu.ccs.com3205.lodchecker.ui;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.swt.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.events.*;
import org.eclipse.ui.dialogs.PropertyPage;
import org.eclipse.ui.plugin.*; 
import org.eclipse.jface.dialogs.*;
import org.eclipse.jface.preference.*;

import org.eclipse.ajdt.ui.AspectJPlugin;

import java.util.*;
import java.io.*;

import edu.neu.ccs.com3205.lodchecker.*;
import edu.neu.ccs.com3205.lodchecker.utils.Messages;
import edu.neu.ccs.com3205.lodchecker.internal.*;

/**
 * LoDCheckerPropertyPage.java
 * This class creates the property page for the LoDCheckerPlugin.  It controls
 * enabling and disabling the checker as well as all of the Checker specific options
 * related to the particular project the property page is displayed for.
 * 
 * @author Paul Freeman
 */

public class LoDCheckerPropertyPage extends PropertyPage
									 implements SelectionListener {

  private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();

	private static WidgetMaker wm = new WidgetMaker();
	private IProject thisProject;			// initialized in createContents
	private Button enableCheckBox;			// enables and disables the LoDChecker
	private Button objectFormButton;		// chooses object form of checker
	private Button classFormButton;		// chooses class form of checker
	private Button avStableCheckBox;		// avoids checking calls to what is defined as stable
	private Button avStaticCheckBox;		// avoids checking calls to static methods	
	private Button stJavaCallCheckBox;		// considers java calls stable
	private Button stJavaMemberCheckBox;	// considers java data member access stable
	private StringFieldEditor customStableTBox;	// text box to enter custom stable packages
	private Button gpStaticCallCheckBox;	// considers static call global preferred
	private Button gpStaticMemberCheckBox;	// considers static data members global preferred
	private StringFieldEditor customGlobalTBox;	// text box to enter custom pointcuts describing global preferred 
		
	private static final boolean ENABLE_DEFAULT = false;
	private static final String FORM_DEFAULT = CheckerProperties.OBJECT_FORM;
	private static final String AVOID_DEFAULT = Checker.ANYD_STABLE + "||" + Checker.ANY_STATIC_CALL;
	private static final String STABLE_DEFAULT = Checker.ANY_JAVA_CALL + "||" + Checker.ANY_JAVA_GET;
	private static final String GLOBAL_DEFAULT = Checker.ANY_STATIC_CALL + "||" + Checker.ANY_STATIC_GET;
	private static final String CUSTOM_DEFAULT = "";

	/**
	 * Constructor for SamplePropertyPage.
	 */
	public LoDCheckerPropertyPage() {
		super();
	}

	/**
	 * @see PreferencePage#createContents(Composite)
	 */
	protected Control createContents(Composite parent) {
		thisProject = (IProject) getElement();
		
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		composite.setLayout(layout);
		GridData data = new GridData(GridData.FILL);
		data.grabExcessHorizontalSpace = false;
		data.grabExcessVerticalSpace = false;
		composite.setLayoutData(data);

		addEnableSection(composite);
		addFormSection(composite);
		addToAvoidSection(composite);
		addStableSection(composite);
		addGlobalPreferredSection(composite);
		addNoteSection(composite);
		
		loadPage();
		
		return composite;
	}

	/**
	 * adds the section which controls the enabling and disabling of the checker
	 */
	private void addEnableSection(Composite parent) {
		Composite composite = wm.createComposite(parent, 2);

		// Descriptive paragraph
		String description = 
		"The Law of Demeter (LoD) Checker is a utility that, " +
		"when enabled, will create a log file containing\n" +
		"the violations of the LoD that were found in the current project.\n";
		Label descriptionText = wm.createLabel(composite, description);
		
		// CheckBox to enable or disable checker
		enableCheckBox = wm.createCheckBox(composite, "Enable the LoD Checker", this);		
	}

	/**
	 * adds the section which controls the form of the LoDChecker to be used
	 */
	private void addFormSection(Composite parent) {
		Composite composite = wm.createComposite(parent, 2);
		addHorizontalSeparator(composite);
			
		//radio button choices between object and class form
		objectFormButton = wm.createRadioButton(composite, "Object Form", this);
		classFormButton = wm.createRadioButton(composite, "Class Form", this);		
	}
	
	/**
	 * adds the section which controls what to avoid while checking a program
	 */
	private void addToAvoidSection(Composite parent) {
		Composite composite = wm.createComposite(parent, 2);
		addHorizontalSeparator(composite);
		
		// Description
		String description = 
		"Avoid Checking:";
		Label descriptionText = wm.createLabel(composite, description);

		// checkboxes for predefined stable pointcuts	
		avStableCheckBox = wm.createCheckBox(composite, "What is defined as Stable", this);		
		avStaticCheckBox = wm.createCheckBox(composite, "Calls to Static Methods", this);		
	}
	
	/**
	 * adds the section that controls what is considered stable when checking
	 * a program
	 */
	private void addStableSection(Composite parent) {
		Composite composite = wm.createComposite(parent, 2);
		addHorizontalSeparator(composite);
		
		// Description
		String description = 
		"Stable:\n" +
		"Choose the calls/data member accesses you wish to define as Stable.\n";
		Label descriptionText = wm.createLabel(composite, description);

		// checkboxes for predefined stable pointcuts	
		stJavaCallCheckBox = wm.createCheckBox(composite, "All Java Package Method Calls", this);		
		stJavaMemberCheckBox = wm.createCheckBox(composite, "All Java Package Data Member Accesses", this);
		
		// text field used to enter custom stable values
		String label = "Enter custom pointcuts. Seperate multiple pointcuts with '||'.";
		Label labelText = wm.createLabel(composite, label);
		Composite tBoxComposite = wm.createComposite(parent, 2);
		customStableTBox = new StringFieldEditor(
									"custStable",
									"",
									StringFieldEditor.UNLIMITED,
									StringFieldEditor.VALIDATE_ON_FOCUS_LOST,
									tBoxComposite);
	}
	
	/**
	 * adds the section that controls what is considered to be a globally preferred object
	 * when checking a program
	 */
	private void addGlobalPreferredSection(Composite parent){
		Composite composite = wm.createComposite(parent, 2);
		addHorizontalSeparator(composite);
			
		// Description
		String description = 
		"Global Preferred:\n" + 
		"Choose the objects you wish to globally allow calls on.";
		Label descriptionText = wm.createLabel(composite, description);

		// checkboxes for predefined global preferred pointcuts	
		gpStaticCallCheckBox = wm.createCheckBox(composite, "Objects returned by all static Method Calls.", this);		
		gpStaticMemberCheckBox = wm.createCheckBox(composite, "All static Data Members.", this);		

		// text field used to enter custom stable values
		String label = "Enter custom pointcuts. Seperate multiple pointcuts with '||'.";
		Label labelText = wm.createLabel(composite, label);
		Composite tBoxComposite = wm.createComposite(parent, 2);
		customGlobalTBox = new StringFieldEditor(
									"custGlobal",
									"",
									StringFieldEditor.UNLIMITED,
									StringFieldEditor.VALIDATE_ON_FOCUS_LOST,
									tBoxComposite);
	}
	
	/**
	 * adds a disclaimer section to the bottom of the page
	 */
	private void addNoteSection(Composite parent) {
		Composite composite = wm.createComposite(parent, 2);
		addHorizontalSeparator(composite);
		
		String note = 
		"NOTE: Violations are checked for dynamically as the program " +
		"executes.. Therefore, all of the violations may not be\n" +
		"found in one execution, and the violations reported may vary between " +
		"program executions.";
		Label noteText = wm.createLabel(composite, note);		
	}

	/**
	 * adds a horizontal separator to the page
	 */
	private Label addHorizontalSeparator(Composite parent) {
		Label separator = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		separator.setLayoutData(gridData);
		return separator;
	}

	/**
	 * populates the form with the preserved values for the current project
	 */
	private void loadPage(){
		setEnabled(
			CheckerProperties.retrieveSettingBoolean(
				thisProject,
				CheckerProperties.ENABLE_PROPERTY,
				ENABLE_DEFAULT));
		
		setFormButtons(
			CheckerProperties.retrieveSettingString(
				thisProject,
				CheckerProperties.FORM_PROPERTY,
				FORM_DEFAULT));
		
		setWhatToAvoid(
			CheckerProperties.retrieveSettingString(
				thisProject,
				CheckerProperties.TO_AVOID_PROPERTY,
				AVOID_DEFAULT));
		
		setWhatStable(
			CheckerProperties.retrieveSettingString(
				thisProject,
				CheckerProperties.WHAT_STABLE_PROPERTY,
				STABLE_DEFAULT));
		setCustomStable(
			CheckerProperties.retrieveSettingString(
				thisProject,
				CheckerProperties.CUST_STABLE_PROPERTY,
				CUSTOM_DEFAULT));
				
		setWhatGlobal(
			CheckerProperties.retrieveSettingString(
				thisProject,
				CheckerProperties.WHAT_GLOBAL_PROPERTY,
				GLOBAL_DEFAULT));
		setCustomGlobal(
			CheckerProperties.retrieveSettingString(
				thisProject,
				CheckerProperties.CUST_GLOBAL_PROPERTY,
				CUSTOM_DEFAULT));
				
	}

	/**
	 * called when the restore defaults button is pressed
	 */
	protected void performDefaults() {
		// Populate the form with the default values
		setEnabled(ENABLE_DEFAULT);
		setFormButtons(FORM_DEFAULT);
		setWhatToAvoid(AVOID_DEFAULT);
		setWhatStable(STABLE_DEFAULT);
		setCustomStable(CUSTOM_DEFAULT);
		setWhatGlobal(GLOBAL_DEFAULT);
		setCustomGlobal(CUSTOM_DEFAULT);
	}
	
	/**
	 * called when either the apply or ok buttons are pressed
	 */
	public boolean performOk() {
		boolean propCheck = false;
		boolean enableCheck = true;
		boolean hadAJNature = false;	// used for removal if error occurrs
		String enableError = "Failed to enable the Law of Demeter Checker";
		String disableError = "Failed to disable the Law of Demeter Checker";
		String errorTitle = "INTERNAL ERROR";
		
		plugin.log("user pressed prop page OK");
		
		// reset builder since form option may have changed
		Builder.resetBuildConfigurationFile(thisProject);

		// store the form values 
		propCheck = storeAllProperties();
		
		// check if this project already has AspectJ Nature
		try{
			if(thisProject.hasNature(AspectJPlugin.ID_NATURE)){
				hadAJNature = true;
			}
		}catch(CoreException e){
			hadAJNature = false;
		}
		
		if(getEnabled()){
		//enabled
			try{
				Checker.enable(thisProject);
				this.setEnabled(true);
				
			}catch(CoreException e){			
				Messages.error(errorTitle, enableError, e);
				plugin.log(enableError, e);
				try{
					Checker.disable(thisProject, !hadAJNature); 
				}catch(CoreException e2){
					Messages.error(errorTitle, disableError, e2);
					plugin.log(disableError, e);
				}

				// here checker could be partially enabled... no way around this
				this.setEnabled(false);
				enableCheck = false;
			}
		}else{
			//not enabled
			try{
				Checker.disable(thisProject);
				this.setEnabled(false);
				
			}catch(CoreException e){
				Messages.error(errorTitle, disableError, e);
				plugin.log(disableError, e);
				
				// keep checker enabled
				this.setEnabled(true);
				enableCheck = false;
			}
		}
		
		// store values incase had to change due to error	
		propCheck = storeAllProperties();

		return (propCheck && enableCheck);
	}

	
	/**
	 * helper function to store the current settings on the properties page
	 */
	private boolean storeAllProperties(){
		try{
			// enable
			CheckerProperties.preserveSetting(
				thisProject,
				CheckerProperties.ENABLE_PROPERTY,
				getEnabled());
				
			// form choice
			CheckerProperties.preserveSetting(
				thisProject,
				CheckerProperties.FORM_PROPERTY,
				getFormProperty());
				
			// avoid choice
			CheckerProperties.preserveSetting(
				thisProject,
				CheckerProperties.TO_AVOID_PROPERTY,
				getWhatToAvoid());
				
			// stable choices
			CheckerProperties.preserveSetting(
				thisProject,
				CheckerProperties.WHAT_STABLE_PROPERTY,
				getWhatStable());
			CheckerProperties.preserveSetting(
				thisProject,
				CheckerProperties.CUST_STABLE_PROPERTY,
				getCustomStable());
				
			// global properties
			CheckerProperties.preserveSetting(
				thisProject,
				CheckerProperties.WHAT_GLOBAL_PROPERTY,
				getWhatGlobal());
			CheckerProperties.preserveSetting(
				thisProject,
				CheckerProperties.CUST_GLOBAL_PROPERTY,
				getCustomGlobal());
				
		} catch (CoreException e) {
			plugin.log("Error storing properties!", e);
			return false;
		}
		return true;
	}

	/**
	 * helper function for symetry to get the enabled setting
	 */
	private boolean getEnabled(){
		return enableCheckBox.getSelection();
	}

	/**
	 * helper function to perform all actions when setting the checker to enabled
	 */
	private void setEnabled(boolean enabled){
		// set the enabled checkbox
		enableCheckBox.setSelection(enabled);
		
		// enable all other controls accordingly
		enableControls();
		
	}
	
	/**
	 * enables the controls based on the current enableCheckBox value
	 */
	private void enableControls(){
		objectFormButton.setEnabled(getEnabled());
		classFormButton.setEnabled(getEnabled());				
		avStableCheckBox.setEnabled(getEnabled());
		avStaticCheckBox.setEnabled(getEnabled());

		enableStableControls();

		gpStaticCallCheckBox.setEnabled(getEnabled());
		gpStaticMemberCheckBox.setEnabled(getEnabled());
//		customGlobalTBox.setEnabled(getEnabled());
	}

	/**
	 * helper function to enable the stable controls based on 
	 * enablecheckbox and toAvoid stable check box
	 */
	private void enableStableControls(){
		stJavaCallCheckBox.setEnabled(getEnabled() && avStableCheckBox.getSelection());
		stJavaMemberCheckBox.setEnabled(getEnabled() && avStableCheckBox.getSelection());
//		customStableTBox.setEnabled(getEnabled() && avStableCheckBox.getSelection());
	}
	
	/**
	 * helper function to set the form radio buttons
	 */
	private void setFormButtons(String form){
		if(form.equalsIgnoreCase(CheckerProperties.CLASS_FORM)){
			objectFormButton.setSelection(false);
			classFormButton.setSelection(true);
		}else{
			objectFormButton.setSelection(true);
			classFormButton.setSelection(false);
		}
	}
	
	/**
	 * helper function to get the form property based on the radio button selection
	 */
	private String getFormProperty(){
		if(classFormButton.getSelection()){
			return CheckerProperties.CLASS_FORM;
		}else{
			return CheckerProperties.OBJECT_FORM;
		}
	}
	
	/**
	 * @return returns a string describing what to avoid
	 */
	private String getWhatToAvoid(){
		String avoid = "";
		boolean avoidSomething = false;
		if(avStableCheckBox.getSelection()){
			avoid += Checker.ANYD_STABLE;
			avoidSomething = true;
		}
		if(avStaticCheckBox.getSelection()){
			if(avoidSomething){ avoid += "||";}
			avoid += Checker.ANY_STATIC_CALL;
			avoidSomething = true;
		}
		if(!avoidSomething){
			avoid = Checker.ANY_NOTHING;
		}
		return avoid;
	}
	
	/**
	 * helper function sets the what to avoid boxes
	 */
	private void setWhatToAvoid(String avoid){
		if(avoid.indexOf(Checker.ANYD_STABLE) >= 0){
			avStableCheckBox.setSelection(true);
		}else{
			avStableCheckBox.setSelection(false);
		}			
		enableStableControls();
		
		if(avoid.indexOf(Checker.ANY_STATIC_CALL) >= 0){
			avStaticCheckBox.setSelection(true);
		}else{
			avStaticCheckBox.setSelection(false);
		}
	}

	/**
	 * @return returns a string describing what is stable
	 */
	private String getWhatStable(){
		String stable = "";
		boolean somethingStable = false;
		if(stJavaCallCheckBox.getSelection()){
			stable += Checker.ANY_JAVA_CALL;
			somethingStable = true;
		}
		if(stJavaMemberCheckBox.getSelection()){
			if(somethingStable){ stable += "||";}
			stable += Checker.ANY_JAVA_GET;
			somethingStable = true;
		}
		if(!somethingStable){
			stable = Checker.ANY_NOTHING;
		}
		return stable;
	}
	
	/**
	 * helper function sets the stable boxes
	 */
	private void setWhatStable(String stable){
		if(stable.indexOf(Checker.ANY_JAVA_CALL) >= 0){
			stJavaCallCheckBox.setSelection(true);
		}else{
			stJavaCallCheckBox.setSelection(false);
		}			
		if(stable.indexOf(Checker.ANY_JAVA_GET) >= 0){
			stJavaMemberCheckBox.setSelection(true);
		}else{
			stJavaMemberCheckBox.setSelection(false);
		}
	}
	
	/**
	 * @return returns the custom stable entry
	 */
	private String getCustomStable(){
		return customStableTBox.getStringValue();
	}
	
	/**
	 * helper function sets the custom stable entry
	 */
	private void setCustomStable(String stable){
		customStableTBox.setStringValue(stable);
	}
	
	/**
	 * @return returns a string describing what is stable
	 */
	private String getWhatGlobal(){
		String global = "";
		boolean somethingGlobal = false;
		if(gpStaticCallCheckBox.getSelection()){
			global += Checker.ANY_STATIC_CALL;
			somethingGlobal = true;
		}
		if(gpStaticMemberCheckBox.getSelection()){
			if(somethingGlobal){ global += "||";}
			global += Checker.ANY_STATIC_GET;
			somethingGlobal = true;
		}
		if(!somethingGlobal){
			global = Checker.ANY_NOTHING;
		}
		return global;
	}
	
	/**
	 * helper function sets the stable boxes
	 */
	private void setWhatGlobal(String global){
		if(global.indexOf(Checker.ANY_STATIC_CALL) >= 0){
			gpStaticCallCheckBox.setSelection(true);
		}else{
			gpStaticCallCheckBox.setSelection(false);
		}			
		if(global.indexOf(Checker.ANY_STATIC_GET) >= 0){
			gpStaticMemberCheckBox.setSelection(true);
		}else{
			gpStaticMemberCheckBox.setSelection(false);
		}
	}
	
	/** 
	 * @return returns the custom stable entry
	 */
	private String getCustomGlobal(){
		return customGlobalTBox.getStringValue();
	}
	
	/**
	 * helper function sets the custom stable entry
	 */
	private void setCustomGlobal(String global){
		customGlobalTBox.setStringValue(global);
	}	
		
////////
// SelectionListener interface starts here
////////
	/**
	 * Handle selection of an item in the menu.
	 */
	public void widgetDefaultSelected(SelectionEvent se) {
		widgetSelected(se);
	}

	/**
	 * Handle selection of an item in the menu.
	 */
	public void widgetSelected(SelectionEvent se) {
		Object source = se.getSource();
		if (source instanceof Button) {
			Button btn = (Button) source;
			// enable the controls according enableCheckBox selection
			if (btn.equals(enableCheckBox)) {
				enableControls();
			}else if(btn.equals(avStableCheckBox)){
				enableStableControls();
			}
		}
	}
	
	

}