package edu.neu.ccs.com3205.lodchecker.ui;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.SWT;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.events.*;
import org.eclipse.ui.*;
import org.eclipse.jface.action.*;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.debug.core.*;
import org.eclipse.jface.resource.*;

import java.util.*;
import java.io.*;

import edu.neu.ccs.com3205.lodchecker.*;
import edu.neu.ccs.com3205.lodchecker.utils.*;
import edu.neu.ccs.com3205.lodchecker.internal.*;

/**
 * LoDCheckerViolationsView.java
 * This class creates the view which displays the violations and statistics for
 * the project currently selected.  It also provides all of the functionality to 
 * monitor selections within the violations display - opening an editor and selecting
 * a line whenever an open event is received for a selected violation.<br>
 * <br>
 * The view also supports two modes.<br>
 * A toggle between showing the target and this objects that correspond to violations.<br>
 * A toggle between running in a light or normal mode.  The light mode shows violations
 * only after a running process has terminated.  The normal mode displays violations while
 * a process is running.
 * NOTE: currently there is a bug in the Eclipse environment that seems to be displacing
 * objects in memory that are being accessed by a seperate thread.  Strange NullPointerExceptions
 * are thrown when attempting to write violations to the display while a process is running.
 * The violations should not occur according as far as I can tell based on the code, leading
 * me to beleive it as bug in the Eclipse environment.  Therefore the light/normal toggle
 * is currently disabled making the light mode permanently activated.
 * 
 * @author Paul Freeman
 */

public class LoDCheckerViolationsView extends ViewPart 
		implements ControlListener, SelectionListener, IOpenListener, IPageListener {
      
	public static final String VIEW_ID = "edu.neu.ccs.com3205.lodchecker.ui.LoDCheckerViolationsView";
	private static final String DISABLED_MESSAGE = "<-- checker is not enabled for this project -->";
	private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();
      
  // control objects
  private static WidgetMaker wm = new WidgetMaker();
  private TabFolder tabs;							// the tabs on the violations display
  private TabItem vTab;								// the violations tab
  private TabItem statTab;							// the statistics tab
  private Table vTable;								// the table of violations
	private TableColumn vCol;						// the column of violations
	private TableColumn vLoc;						// the column of locations
	private TableColumn vThis;						// the column of this objects
	private TableColumn vTarget; 				// the column of target objects
	private TableColumn vCountCol;				// the column of counts for violations
	private TableViewer vViewer;					// the violation table viewer
	private Table statTable;							// the table of statistics
	
	// actions
	private Action toggleThisAction;			// toggles showing the this and target objects
	private boolean showThisTarget;			// switch for toggle of this/target column
	private static final String SHOW_THIS_TARGET_KEY = "showThisTarget";
	private static final String TOGGLE_TEXT = "Show 'this' & 'target'";
	private static final String TOGGLE_TOOLTIP = TOGGLE_TEXT;
	private static final String TOGGLE_ON_IMAGE = "thisTarget_on.gif";
	private static final String TOGGLE_OFF_IMAGE = "thisTarget_off.gif";
	private static ImageDescriptor toggleThisOn_ID;
	private static ImageDescriptor toggleThisOff_ID;

	private Action toggleLightAction;		// toggles the reporting mode of the view
	private boolean inLightMode;				// switch for toggle of light/normal mode
	private static final String LIGHT_NORMAL_KEY = "lightNormal";
	private static final String LIGHT_TEXT = "Light Feedback Mode";
	private static final String LIGHT_TOOLTIP = "Enable Light Feedback Mode";
	private static final String LIGHT_MODE_MESSAGE = "Light Mode: display will update upon termination...";
	private static final String LIGHT_ON_IMAGE = "lightMode_on.gif";
	private static final String LIGHT_OFF_IMAGE = "lightMode_off.gif";
	private static ImageDescriptor toggleLightOn_ID;
	private static ImageDescriptor toggleLightOff_ID;
	
	// tab variables
	private static final int VTAB_INDEX = 0;
	private static final String VTAB_TEXT = "Violations";
	private static final String VTAB_TOOLTIP = "Violations for the last run of ";
	private static final int STAT_TAB_INDEX = 1;
	private static final String STAT_TAB_TEXT = "Statistics";
	private static final String STAT_TAB_TOOLTIP = "Statistics for last run of ";
	
	// table columns variables
	private static final int VCOL_INDEX = 0;
	private static final int VLOC_INDEX = 1;
	private static final int VTHIS_INDEX = 2;
	private static final int VTARGET_INDEX = 3;
	private static final int VCOUNT_INDEX = 4;
	private static final String VCOL_HEAD = "Violation";
	private static final String VLOC_HEAD = "Location";
	private static final String VTHIS_HEAD = "'this'";
	private static final String VTARGET_HEAD = "'target'";
	private static final String VCOUNT_HEAD = "Count";
      
  // violation storage
  private HashMap violations = new HashMap();
  private IProject currentProject = null;
  
  // page variables
  private static LoDCheckerViolationsView activeView = null;	// used to return the active view in getActiveView
  private static Display viewDisplay = null;
  
  /**
   * constructor
   */
  public LoDCheckerViolationsView() {
  	showThisTarget = getShowThisTarget();  	
  	inLightMode = getInLightMode();
  	
  	// monitor changes in the active page
  	plugin.getWorkbench().getActiveWorkbenchWindow().addPageListener(this);
  }
      
	/**
	 * called when the view is disposed
	 */
	public void dispose(){
		activeView = null;
	}
	
  /**
   * called when the view is initialized
   */
  public void createPartControl(Composite parent) {
   	this.setTitle("Law of Demeter Violations");  	
	
		// create widgets
	 	tabs = new TabFolder(parent, SWT.NONE);
  	createViolationsTab(tabs);
  	createStatsTab(tabs);
  	loadLocalToolBar(getViewSite().getActionBars().getToolBarManager());

		// add listener to widgets
  	tabs.addSelectionListener(this);
  	vViewer.addOpenListener(this);
  	
  	// store the current view now that it has been made visible
  	activeView = this;
  	viewDisplay = Display.getCurrent();
  	
  	// load the violations for the current project if possible
  	currentProject = LoDSelectionListener.getCurrentProject();
  	if(currentProject != null){
  		loadViolations(currentProject, true);
  	}
	}

	/**
	 * helper function that creates the violations tab
	 */
	private void createViolationsTab(TabFolder tabs){
		vTab = new TabItem(tabs, SWT.NONE, VTAB_INDEX);
		vTab.setText(VTAB_TEXT);
		vTab.setToolTipText(VTAB_TOOLTIP);
		
		vTab.setControl(createViolationsTable(vTab.getParent()));
	}
	
	/**
	 * helper function that creates the violations tab
	 */
	private void createStatsTab(TabFolder tabs){
		statTab = new TabItem(tabs, SWT.NONE, STAT_TAB_INDEX);
		statTab.setText(STAT_TAB_TEXT);
		statTab.setToolTipText(STAT_TAB_TOOLTIP);
		
		statTab.setControl(createStatsTable(statTab.getParent()));
	}
	
	/**
	 * helper function that creates the violations table
	 */
	private Table createViolationsTable(Composite parent){
 		// create the violation table
 		vTable = new Table(parent, SWT.SINGLE | SWT.FULL_SELECTION);
 		vTable.setHeaderVisible(true);
		
		// create violation column
		vCol = new TableColumn(vTable, SWT.LEFT, VCOL_INDEX);
		vCol.setText(VCOL_HEAD);
		vCol.setResizable(true);
				
		// create location column
		vLoc = new TableColumn(vTable, SWT.LEFT, VLOC_INDEX);
		vLoc.setText(VLOC_HEAD);
		vLoc.setResizable(true);
		
		// create this column
		vThis = new TableColumn(vTable, SWT.LEFT, VTHIS_INDEX);
		vThis.setText(VTHIS_HEAD);
		vThis.setResizable(true);
		
		// create target column
		vTarget = new TableColumn(vTable, SWT.LEFT, VTARGET_INDEX);
		vTarget.setText(VTARGET_HEAD);
		vTarget.setResizable(true);
		
		// create the violation count column
		vCountCol = new TableColumn(vTable, SWT.CENTER, VCOUNT_INDEX);
		vCountCol.setText(VCOUNT_HEAD);
		vCountCol.setResizable(true);
				
		// register this view class as a listener for the violations table
		vTable.addControlListener(this);
		
		// set the viewer for the table
		vViewer = new TableViewer(vTable);
		
		return vTable;
	}

	/**
	 * helper function that creates the Statistics table
	 */
	private Table createStatsTable(Composite parent){
		statTable = new Table(parent, SWT.SINGLE | SWT.FULL_SELECTION);
		statTable.setHeaderVisible(true);
		
		return statTable;
	}
	
	/**
	 * called whenever the view receives focus
	 */
  public void setFocus() {
    // set focus to my widget.  For a label, this doesn't
    // make much sense, but for more complex sets of widgets
    // you would decide which one gets the focus.
  }

	/**
	 * loads the toolbar for the view with the action buttons
	 */
	private void loadLocalToolBar(IToolBarManager manager){
		makeActions();
		manager.add(toggleThisAction);
		manager.add(toggleLightAction);
	}
		
	/**
	 * clears the violations display
	 */
	synchronized private void resetDisplay(){
		// clear vTable
		vTable.removeAll();
		violations = new HashMap();
		
		// clear statTBox
		statTable.removeAll();
		resizeStatTable();
	}
	
	/**
	 * adds the statement to the appropriate display
	 */
	synchronized public void addToDisplay(String statement, boolean allDisplays){
		if(allDisplays){
			// add message to vTable
			TableItem tableItem = new TableItem(vTable, SWT.NONE);
			tableItem.setText(VCOL_INDEX, statement);
			vTable.update();
			
			// add message to statTBox
			tableItem = new TableItem(statTable, SWT.NONE);
			tableItem.setText(VCOL_INDEX, statement);
			statTable.update();
		}else{
			if(statement.startsWith(Checker.V_TAG)){
				addViolation(statement);
			}else{
				// currently anything that is not a violation is considered a statistic
				addStatistic(statement);
			}
		}
	}
	
	/**
	 * adds the given violation to the table increasing the occurrence count
	 * of the violation by one
	 */
	private void addViolation(String violation){
		// get the violation item from the table corresponding to this violation
		TableItem vTableItem = getViolationItem(violation);

		// increase count
		long count = new Long(vTableItem.getText(VCOUNT_INDEX)).longValue() + 1;
		vTableItem.setText(VCOUNT_INDEX, new Long(count).toString());

		// store the changed violation item
		if(showThisTarget){
			// store violation by complete violation string to count only where target and this match
			storeViolationItem(violation, vTableItem);
		}else{
			// store based on parsed violation to count only occurrences of violation line
			storeViolationItem(vTableItem.getText(VCOL_INDEX), vTableItem);
		}
		vTable.update();
	}
	
	/**
	 * adds the given statistic to the statistics display
	 */
	private void addStatistic(String stat){
		if(!stat.trim().equalsIgnoreCase("")){
			Vector statElements = parseStatistic(stat);
			
			// make sure there are enough columns to hold new stat
			int colCount = statTable.getColumnCount();
			int numElements = statElements.size();
			if(colCount < numElements){
				for(int i = colCount; i < statElements.size(); i++){
					TableColumn col = new TableColumn(statTable, SWT.LEFT, i);
					col.setResizable(true);
				}
				resizeStatTable();
				statTable.redraw();
			}
					
			// add the new statistic
			TableItem tItem = new TableItem(statTable, SWT.NONE);
			for(int j = 0; j < statElements.size(); j++){
				tItem.setText(j, (String)statElements.elementAt(j));
			}
			statTable.update();
		}
	}
	
	/**
	 * helper function that parses a statistic string 
	 * @return returns the elements of the string as a Vector
	 */
	private Vector parseStatistic(String stat){
		//create new table item with violation intialized at count 0
		Vector elements = new Vector();
		StringTokenizer statElements = new StringTokenizer(stat, "\t");

		while(statElements.hasMoreElements()){
			elements.add(statElements.nextElement());
		}
		
		return elements;
	}
	
	/**
	 * @return returns the TableItem from the ViolationsTable
   * which corresponds to the given violation String.
   */
	private TableItem getViolationItem(String violation){
		// remove the tag from the beginning of the violation
		violation = violation.substring(Checker.V_TAG.length()).trim();
		Vector vElements = parseViolation(violation);
		
		TableItem vTableItem = (TableItem)violations.get(vElements.elementAt(VCOL_INDEX));
		
		if(vTableItem == null){
			//create new table item with violation intialized at count 0
			vTableItem = createViolationItem(vElements);
		}
		return vTableItem;
	}
	
	/**
	 * helper function that creates a violation TableItem
	 * @return returns a new violations table item filled with the elements in vElements
	 */
	private TableItem createViolationItem(Vector vElements){
		//create new table item with violation intialized at count 0
		TableItem vTableItem = new TableItem(vTable, SWT.NONE);
		vTableItem.setText(VCOUNT_INDEX, "0");
		for(int i = 0; i < vElements.size(); i++){
			vTableItem.setText(i, (String)vElements.elementAt(i));
		}
		
		return vTableItem;
	}

	/**
	 * helper function that parses a violation
	 * @return returns a Vector containing the parsed elements of the violation
	 */
	private Vector parseViolation(String violation){
		//create new table item with violation intialized at count 0
		Vector elements = new Vector();
		StringTokenizer vElements = new StringTokenizer(violation);
		String element = "";
		String thisElement = "";
		while(vElements.hasMoreElements()){
			while(vElements.hasMoreElements() && 
				!isViolationDelimiter(thisElement = (String)vElements.nextElement())){			
					element += (" " + thisElement);
			}
			int thisIndex = priorColumn(thisElement);
			if(thisIndex >= elements.size()){ elements.setSize(thisIndex + 1); }
			elements.setElementAt(element, thisIndex);
			element = "";
		}
		
		return elements;
	}
		
	/**
	 * @return returns true if this element is a violation delimiter
	 * and false otherwise
	 */
	private boolean isViolationDelimiter(String delim){
		if(delim.equalsIgnoreCase("at") ||
				delim.equalsIgnoreCase("this") ||
				delim.equalsIgnoreCase("target")){
			return true;
		}else{
			return false;
		}
	}
	
	/**
	 * @return returns the index of the column before the given delimiter
	 */
	private int priorColumn(String delim){
		if(delim.equalsIgnoreCase("at")){
			return VCOL_INDEX;
		}else if(delim.equalsIgnoreCase("this")){
			return VLOC_INDEX;
		}else if(delim.equalsIgnoreCase("target")){
			return VTHIS_INDEX;
		}else{
			return VTARGET_INDEX;
		}
	}
	
	/**
	 * helper function that stores the violation item
	 */
	private void storeViolationItem(String violation, TableItem vTableItem){
		violations.put(violation, vTableItem);
	}
	
////////
// ControlListener interface starts here
////////

	/**
	 * Sent when the location (x, y) of a control 
	 * changes relative to its parent 
	 * (or relative to the display, for Shells).
	 */
	public void controlMoved(ControlEvent e){
	}
	
	/**
	 * sent when the size (width/height) of a control changes
	 */
	public void controlResized(ControlEvent e){
		resizeVTable();	
		resizeStatTable();	
	}
//////////
/// end control listener interface
//////////
	
	/**
	 * helper function that resizes the violations table based on the value of
	 * the showThisTarget toggle
	 */
	private void resizeVTable(){
		int colWidth = getVColWidth();
		vCol.setWidth(colWidth);
		vLoc.setWidth(colWidth);
		vCountCol.setWidth(colWidth);
		
		if(getShowThisTarget()){
			vThis.setWidth(colWidth);
			vTarget.setWidth(colWidth);
		}else{
			vThis.setWidth(0);
			vTarget.setWidth(0);
		}
	}
	
	/**
	 * @return returns a calculated width for all violations table columns
	 */
	private int getVColWidth(){
		double vTableW = (double)vTable.getBounds().width;
		double numCols = (double)vTable.getColumnCount();
		if(!getShowThisTarget()){ numCols = numCols - 2;}
		double percent = (1 / numCols);
		return new Long(Math.round(percent * vTableW)).intValue();
	}

	/**
	 * helper function that resizes the statistics table
	 */
	private void resizeStatTable(){
		int colWidth = getStatColWidth();
		int colCount = statTable.getColumnCount();
		for(int i = 0; i < colCount; i++){
			statTable.getColumn(i).setWidth(colWidth);
		}
	}
	
	/**
	 * @return returns a calculated width for all statistics table columns
	 */
	private int getStatColWidth(){
		double numCols = (double)statTable.getColumnCount();
		double percent = .20;
		if(numCols > 0){
			percent = (1 / numCols);
		}
		double statTableW = (double)statTable.getBounds().width;
		return new Long(Math.round(percent * statTableW)).intValue();
	}

	/**
	 * Function will display the violations view in the current perspective
	 */
	synchronized public static void showView() throws PartInitException{
		plugin.log("showing violations view");
		
		IWorkbenchWindow thisWindow = plugin.getDefault().getWorkbench().getActiveWorkbenchWindow();
		if(thisWindow == null){
			plugin.log("active window is null");
		}else{
			IViewPart thisView = thisWindow.getActivePage().findView(VIEW_ID);
			plugin.log("showing view on " + thisWindow.getActivePage().getPerspective().getLabel());
			if(thisView == null){
				IWorkbenchPart curPart = thisWindow.getActivePage().getActivePart();
				thisWindow.getActivePage().showView(VIEW_ID);
				plugin.log("view activated on " + thisWindow.getActivePage().getPerspective().getLabel());
				curPart.setFocus();
			}else{
				plugin.log("violations view was already visible");
			}
		}
	}
	
	/**
	 * Function that will hide the violations view in the current perspective
	 */
	synchronized public static void hideView(){
		plugin.log("hiding violations view");
		
		IWorkbenchWindow thisWindow = plugin.getDefault().getWorkbench().getActiveWorkbenchWindow();
		if(thisWindow == null){
			plugin.log("active window is null");
		}else{
			IViewPart thisView = thisWindow.getActivePage().findView(VIEW_ID);
			plugin.log("hiding view on " + thisWindow.getActivePage().getPerspective().getLabel());
			if(thisView == null){
				plugin.log("view is not visible");
			}else{
				IWorkbenchPart curPart = thisWindow.getActivePage().getActivePart();
				thisWindow.getActivePage().hideView(thisView);
				plugin.log("view hidden on " + thisWindow.getActivePage().getPerspective().getLabel());
				curPart.setFocus();
			}
		}
	}

	/**
	 * Loads the violations for the given project.  If the project is not LoDChecker enabled
	 * an appropriate message is displayed.  If force is true then the violations are 
	 * forced to reload, if false, the violations may not reload unless the project has
	 * changed
	 */
	synchronized public void loadViolations(IProject project, boolean force){
		if(project != null && (force || project != currentProject)){
			plugin.log("loading violations for project: " + project);
			boolean projEnabled = setCurrentProject(project);
															
			if(projEnabled){
				resetDisplay();

				//load the violations for this project if any
				File vFile = plugin.getTempViolationsFile(project).toFile();
				if(vFile.exists()){
					try{
						BufferedReader vFileIn = new BufferedReader(new FileReader(vFile));
						String nextLine;
						while((nextLine = vFileIn.readLine()) != null){
							addToDisplay(nextLine, false);
						}
						vFileIn.close();
					}catch(IOException e){
						plugin.log("Failed to read violations file: " + vFile, e);
					}
				}else{
					plugin.log("Violation file does not exist: " + vFile);
				}
			}else{
				showDisabled();
			}
		}		
	}
	
	/**
	 * @return Returns the currently visible instance of itself or null if one is not available
	 */
	synchronized public static LoDCheckerViolationsView getActiveView(){
		return activeView;
	}
	
	/**
	 * Sets the stored currentProject for the selected project in the workspace
	 * @return returns true if the checker is enabled for this project and false otherwise
	 */
	private boolean setCurrentProject(IProject project){
		boolean projEnabled = Checker.isEnabled(project);
		currentProject = project;
			
		String projName = project.getName();
		vTab.setToolTipText(VTAB_TOOLTIP + projName);
		statTab.setToolTipText(STAT_TAB_TOOLTIP + projName);
		return projEnabled;		
	}

	/**
	 * shows in the display that the checker is disabled for this project
	 */
	private void showDisabled(){
		resetDisplay();
		
		addToDisplay(DISABLED_MESSAGE, true);
	}
	
	/**
	 * shows in the display that the checker is disabled for this project
	 */
	synchronized public void initializeDisplay(){
		resetDisplay();

		if(getInLightMode()){		
			addToDisplay(LIGHT_MODE_MESSAGE, true);
		}
	}
	
	/**
	 * @return returns true if display is showing the target and this columns and 
	 * false otherwise
	 */
	synchronized public boolean getShowThisTarget(){
		return plugin.getPreferenceStore().getBoolean(SHOW_THIS_TARGET_KEY);
	}
	
	/**
	 * toggles the visibility of the target and this columns
	 */
	synchronized public void toggleThisTarget(){
		// toggle column width
		showThisTarget = !getShowThisTarget();
		plugin.log("showing 'this'/'target' objects: " + showThisTarget);
		
		// toggle the toolbar button - if called from button this will have no effect
		toggleThisAction.setChecked(showThisTarget);
		if(showThisTarget){
			if(toggleThisOn_ID != null){
				toggleThisAction.setImageDescriptor(toggleThisOn_ID);
			}
		}else{
			if(toggleThisOff_ID != null){
				toggleThisAction.setImageDescriptor(toggleThisOff_ID);
			}
		}

		// save the new value
		plugin.getPreferenceStore().putValue(SHOW_THIS_TARGET_KEY, new Boolean(showThisTarget).toString());
		plugin.savePluginPreferences();

		// force the violations to reload
		loadViolations(currentProject, true);
		resizeVTable();
	}

	/**
	 * @return returns true if display is showing the target and this columns and 
	 * false otherwise
	 */
	synchronized public boolean getInLightMode(){
		return plugin.getPreferenceStore().getBoolean(LIGHT_NORMAL_KEY);
	}
	
	/**
	 * toggles the visibility of the target and this columns
	 */
	synchronized public void toggleLight(){
		// toggle column width
		inLightMode = !getInLightMode();
		
		// toggle the toolbar button - if called from button this will have no effect
		toggleLightAction.setChecked(inLightMode);
		if(inLightMode){
			if(toggleLightOn_ID != null){
				toggleLightAction.setImageDescriptor(toggleLightOn_ID);
			}
		}else{
			if(toggleLightOff_ID != null){
				toggleLightAction.setImageDescriptor(toggleLightOff_ID);
			}
		}

		
		// save the new value
		plugin.getPreferenceStore().putValue(LIGHT_NORMAL_KEY, new Boolean(inLightMode).toString());
		plugin.savePluginPreferences();
		plugin.log("light mode: " + inLightMode);
	}

	/**
	 * creates all of the appropriate action buttons
	 */
	private void makeActions(){
		plugin.log("making action objects...");
		
		// get the location of the images for the actions
		IPath imagePath = new Path(plugin.getAbsolutePath()).append(plugin.ICONS_DIR);
		
		// action to toggle between showing and not showing this/target objects
		toggleThisAction = new Action() {
			public int getStyle () { 
				return IAction.AS_CHECK_BOX; 
			}
			public void run () {
				plugin.log("this/target button received toggle");
				LoDCheckerViolationsView.getActiveView().toggleThisTarget();
			}
		};
		toggleThisAction.setText(TOGGLE_TEXT); 
		toggleThisAction.setToolTipText(TOGGLE_TOOLTIP);
		toggleThisAction.setChecked(getShowThisTarget());
		try{
			toggleThisOn_ID = ImageDescriptor.createFromURL(imagePath.append(TOGGLE_ON_IMAGE).toFile().toURL()); 
			toggleThisOff_ID = ImageDescriptor.createFromURL(imagePath.append(TOGGLE_OFF_IMAGE).toFile().toURL()); 
			if(getShowThisTarget()){		
				toggleThisAction.setImageDescriptor(toggleThisOn_ID);
			}else{
				toggleThisAction.setImageDescriptor(toggleThisOff_ID);
			}
		}catch(Exception e){
			plugin.log("failed to create toggle images", e);
		}
		
		// action to toggle between light and run mode
		toggleLightAction = new Action() {
			public int getStyle () { 
				return IAction.AS_CHECK_BOX; 
			}
			public void run () {
				plugin.log("light mode button received toggle");
				LoDCheckerViolationsView.getActiveView().toggleLight();
			}
		};
		toggleLightAction.setText(LIGHT_TEXT); 
		toggleLightAction.setToolTipText(LIGHT_TOOLTIP);
		plugin.log("setting inLightMode to: " + getInLightMode());
		toggleLightAction.setChecked(getInLightMode());
		plugin.log("inLightMode checked: " + toggleLightAction.isChecked());
		try{
			toggleLightOn_ID = ImageDescriptor.createFromURL(imagePath.append(LIGHT_ON_IMAGE).toFile().toURL()); 
			toggleLightOff_ID = ImageDescriptor.createFromURL(imagePath.append(LIGHT_OFF_IMAGE).toFile().toURL()); 
			if(getInLightMode()){		
				toggleLightAction.setImageDescriptor(toggleLightOn_ID);
			}else{
				toggleLightAction.setImageDescriptor(toggleLightOff_ID);
			}
		}catch(Exception e){
			plugin.log("failed to create toggle images", e);
		}
	}
		
////////
// SelectionListener interface starts here
////////

	/**
	 * Sent when default selection occurs in the control. 
	 */
	public void widgetDefaultSelected(SelectionEvent e) {
		widgetSelected(e);
	}

	/**
	 * Sent when selection occurs in the control. 
	 */
	public void widgetSelected(SelectionEvent e) {
		plugin.log("Selected: " + e.item);
		
		if(e.item instanceof TabItem){
			TabItem tab = (TabItem)e.item;
			if(tab.equals(vTab) || tab.equals(statTab)){
				loadViolations(currentProject, true);
			}
		}
	}
	
/////////
// IOpenListener interface starts here
/////////

	/**
	 * Notifies of an open event. 
	 */
	public void open(OpenEvent event) {
		plugin.log("received open event from " + event.getSource());
		
		// violations table open events
		if(event.getSource().equals(vViewer)){
			TableItem[] rows = vViewer.getTable().getSelection();
			for(int r = 0; r < rows.length; r++){
				TableItem row = rows[r];
				selectViolation(row.getText(VLOC_INDEX));				
			}
		}
	}
	
/////////
// End IOpenListener interface
/////////

	/**
	 * Opens an editor (if necessary) of the file containing the violation and
	 * selects the line of the violation.
	 */
	private void selectViolation(String location){
		StringTokenizer lElements = new StringTokenizer(location, ":");
		
		// if at least the file name and row number was found
		if(lElements.countTokens() >= 2){
			String fileName = lElements.nextToken().trim();	// file name is first
			String rowNumber = lElements.nextToken().trim();	// row number is second
			
			// open editor and select the row
			IFile file = Files.getFileInDefaultClasspath(currentProject, fileName);
			if(file != null){
				IEditorPart editor = openJavaEditor(file);
				if(editor != null){
					IMarker marker = getViolationMarker(file, new Integer(rowNumber).intValue());
					if(marker != null){
						editor.gotoMarker(marker);
					}else{
						plugin.log("failed to select: couldn't create marker");
					}
				}else{
					plugin.log("failed to select: couldn't open editor");
				}
			}else
				plugin.log("failed to select: couldn't find file...");
		}else{
			plugin.log("invalid location string: " + location);
		}
	}
	
	/**
	 * helper function to open an editor for the corresponding IFile
	 * @return returns the editor that was just opened or null if none was opened
	 */
	private IEditorPart openJavaEditor(IFile file){
		try{			
				return plugin.getWorkbench()
											.getActiveWorkbenchWindow()
											.getActivePage()
											.openEditor(file);
		}catch(Exception e){
			plugin.log("openJavaEditor failed to initialize editor part", e);
		}
		return null;
	}
	
	/**
	 * @return returns a Violation marker for the line of line number in a given resource
	 * returns null if a marker could not be created
	 */
	private IMarker getViolationMarker(IResource resource, int lineNum){

		IMarker marker = null;
		try {
				marker = resource.createMarker(IMarker.MARKER);
				marker.setAttribute(IMarker.LINE_NUMBER, lineNum);
		}
		catch (CoreException coreEx) {
			plugin.log("Failed to get marker for line " + lineNum + " in resource " + resource);
		}
		return marker;
	}
	
/////////
// IPageListener interface starts here
/////////

	/**
	 * Notifies this listener that the given page has been activated. 
	 */
	public void pageActivated(IWorkbenchPage page) {
		activeView = (LoDCheckerViolationsView)page.findView(VIEW_ID);
  	viewDisplay = Display.getCurrent();
		plugin.log("page activated - active view now: " + activeView);
	}

	/**
	 * Notifies this listener that the given page has been closed. 
	 */
	public void pageClosed(IWorkbenchPage page) {
		activeView = (LoDCheckerViolationsView)page.findView(VIEW_ID);
  	viewDisplay = Display.getCurrent();
		plugin.log("page closed - active view now: " + activeView);
	}

	/**
	 * Notifies this listener that the given page has been opened. 
	 */
	public void pageOpened(IWorkbenchPage page) {
		// do nothing
	}

	public static Display getViewDisplay(){
		return viewDisplay;
	}
}
