package edu.neu.ccs.com3205.lodchecker.utils;

import org.eclipse.core.resources.*;
import org.eclipse.jdt.core.*;
import org.eclipse.core.runtime.*;

import org.aspectj.ajde.Ajde;

import java.io.*;
import java.util.*;

import edu.neu.ccs.com3205.lodchecker.LoDCheckerPlugin;
import edu.neu.ccs.com3205.lodchecker.ui.*;

/**
 * Classpath.java
 * This class provides static functions revolving around adding and removing 
 * classpath entries from a project.
 * 
 * @author Paul Freeman
 */
public class Classpath {
	
  private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();
	private static String AJDE_PLUGIN_ID = "org.aspectj.ajde";
	
	/**
	 * adds the provided entries to the classpath of the project
	 * if the classpath entry already exists it is not added
	 * @return Returns true if it was successfull or the classpath
	 * already exists and false otherwise.
	 */
	synchronized public static void addToClasspath(IProject project, IClasspathEntry[] newEntries) throws CoreException {
		plugin.log("adding new classpath entries " + Arrays.asList(newEntries));
		
		try{
			IJavaProject myJavaProject = JavaCore.create(project);
			
			// get current classpaths
			Vector classPaths = new Vector();
			classPaths.addAll(Arrays.asList(myJavaProject.getRawClasspath()));
			
			// add new classpaths if they don't already exist
			for(int i = 0; i < newEntries.length; i++){
				if(!classPaths.contains(newEntries[i])){
					classPaths.add(newEntries[i]);
				}
			}
			
			IClasspathEntry[] newClassPaths = (IClasspathEntry[])classPaths.toArray(new IClasspathEntry[0]);
			if(!myJavaProject.hasClasspathCycle(newClassPaths)){
				myJavaProject.setRawClasspath(newClassPaths, new NullProgressMonitor());
			}
		}catch(JavaModelException e){
			String message = "Failed to add classpaths " + Arrays.asList(newEntries) + " to: " + project;
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
		}
	}
	
	/**
	 * removes the provided entries from the classpath of the project.
	 * If the classpath entry did not exist no error is thrown
	 * @return Returns true if it was successfull or the classpath entry
	 * did not exist and false otherwise
	 */
	synchronized public static void removeFromClasspath(IProject project, IClasspathEntry[] oldEntries) throws CoreException {
		plugin.log("removing classpath entries " + Arrays.asList(oldEntries));
		
		try{
			IJavaProject myJavaProject = JavaCore.create(project);
			
			Vector classPaths = new Vector();
			classPaths.addAll(Arrays.asList(myJavaProject.getRawClasspath()));
			classPaths.removeAll(Arrays.asList(oldEntries));
			
			IClasspathEntry[] newClassPaths = (IClasspathEntry[])classPaths.toArray(new IClasspathEntry[0]);
			if(!myJavaProject.hasClasspathCycle(newClassPaths)){
				myJavaProject.setRawClasspath(newClassPaths, new NullProgressMonitor());
			}
		}catch(JavaModelException e){
			String message = "Failed to remove classpaths " + Arrays.asList(oldEntries) + " from: " + project;
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
		}		
	}	
	
	/**
	 * @return returns the absolute IPath to the default classpath folder, 
	 * i.e. the source folder, in the given project
	 * returns null if the classpath cannot be determined
	 */
	synchronized public static IPath getDefaultClasspath(IProject project){
		try{
			//get the classpath entries for the project
			IJavaProject javaProject = JavaCore.create(project);
	    IClasspathEntry[] classpathArray = javaProject.getRawClasspath();
	    
	    // the first classpath entry contains the root source folder in the project
	    String classpath = classpathArray[0].toString();
	    
	    // remove the junk in the entry
	    int index = classpath.indexOf('[');
	    classpath = classpath.substring(0,index).trim();
	    classpath = classpath.substring(project.getName().length() + 1);
	    
	    // return the absolute path to the folder in the project
	    
			return project.getLocation().makeAbsolute().append(classpath);
		}catch(Exception e){
			plugin.log("failed to get the default classpath for: " + project, e);
		}
		return null;
	}
}
