package edu.neu.ccs.com3205.lodchecker.utils;

import java.io.*;
import java.util.*;

import org.aspectj.ajde.*;
import org.eclipse.core.runtime.*;
import org.eclipse.core.resources.*;

import edu.neu.ccs.com3205.lodchecker.*;

/**
 * Files.java
 * This class contains common static functions for manipulating files.
 * 
 * @author Paul Freeman
 */
public class Files {

  private static LoDCheckerPlugin plugin = LoDCheckerPlugin.getDefault();

	/**
	 *  copies all of the files and directories in the java.io.File denoted by from
	 *  to the java.io.File denoted by to.  If force is true then existing files
	 *  will be overwritten.  If force is false then existing files will not be
	 *  overwritten.
	 * @return Returns true if all of the files were successfully copied and 
	 * false otherwise.
	 */
	synchronized public static void copyAll(File from, File to, boolean force) throws CoreException {
		try{
			// get list of files
			File[] fileList = from.listFiles();
			File originalTo = to;
			
			// copy everything
			for(int f = 0; f < fileList.length; f++){
				File thisFile = fileList[f];
				to = new File(originalTo.getAbsolutePath() + File.separator + thisFile.getName());
				
				if(thisFile.isDirectory()){
					copyAll(thisFile, to, force);
					
				}else{
					copyFile(thisFile, to, force);
				}
			}
		}catch(Exception e){
			String message = "Failed to copy all from: '"+from+"' to: '"+to+"'";
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
		}
	}
	
	/**
	 *  copies a file from one location to another
	 *  @return Returns true if the file was copied successfully and false otherwise
	 */
	synchronized public static void copyFile(File from, File to) throws CoreException {
		copyFile(from, to, false);
	}
	
	/**
	 * copies a file from one location to another
	 * if force is true then an existing file will be overwritten, if force is false
	 * then the existing file will not be overwritten
	 * @return Returns true if the file was copied successfully and false otherwise
	 */
	synchronized public static void copyFile(File from, File to, boolean force) throws CoreException {
		try{
			if(!to.exists() || force){
				createNewFile(to);
				BufferedReader fileIn = new BufferedReader(new FileReader(from));
				BufferedWriter fileOut = new BufferedWriter(new FileWriter(to));
				String nextLine;
				while((nextLine = fileIn.readLine()) != null){
					fileOut.write(nextLine);
					fileOut.newLine();
				}
				fileOut.close();
				fileIn.close();
			}
		}catch(Exception e){
			String message = "Failed to copy file from: '"+from+"' to: '"+to+"'";
			plugin.log(message, e);
			throw new CoreException(
				new Status(
					Status.ERROR,
					LoDCheckerPlugin.PLUGIN_ID,
					Status.OK,
					message,
					e));
		}
	}
	
	/**
	 * Creates a file.
	 * Deletes the new file if it already exists and creates a new empty file.
	 * @return Returns true if the new file was successfully created and false
	 * otherwise.
	 */
	synchronized public static boolean createNewFile(File file){
		if(!deleteFile(file)){
			plugin.log("Delete file returned false");
			return false;
		}
		try{
			file.getParentFile().mkdirs();
			if(!file.createNewFile()){
				plugin.log("CreateNewFile returned false");
				return false;
			}
			return true;
		}catch(java.io.IOException e){
			plugin.log("Failed to create new file: \n" + file +"'", e);
			return false;
		}
	}

	/** 
	 * Deletes a file
	 * @return Returns true if the file didn't exist or 
	 * was deleted and false otherwise
	 */
	synchronized public static boolean deleteFile(File file){
		if(file.exists()){
			if(!file.delete()){
				plugin.log("DeleteFile returned false");
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Deletes all of the files and directories in the file denoted by from.
	 * @return Returns true if all of the files were successfully created
	 * and false otherwise.
	 */
	synchronized public static boolean deleteAll(File from){
		try{
			// get list of files
			File[] fileList = from.listFiles();
			
			// copy everything
			for(int f = 0; f < fileList.length; f++){
				File thisFile = fileList[f];
				
				if(thisFile.isDirectory()){
					deleteAll(thisFile);
				}
				deleteFile(thisFile);
			}
			
			return true;
		}catch(Exception e){
			plugin.log("Failed to delete all \nfrom: '"+from+"'", e);
			return false;
		}
	}
	
	/**
	 * This function is expected to be called with two absolute paths.  The 
	 * behavior is undefined if it is called with either or both paths being
	 * relative.
	 * @return Returns a relative path from the fromPath to the toPath
	 */
	synchronized public static String findRelativePath(IPath fromPath, IPath toPath){
		int commonSegments = toPath.matchingFirstSegments(fromPath);
		int segmentsToBackTrack = fromPath.segmentCount() - commonSegments;
		
		// make a path with all of the up directory commands
		String relativePath = "";
		for(int i = 0; i < segmentsToBackTrack; i++){
			relativePath += ".." + File.separator;
		}
		
		// have to remove the device id ourselves
		String deviceId = toPath.getDevice();
		String pathToAdd = toPath.removeFirstSegments(commonSegments).toOSString();
		if(deviceId != null){
			pathToAdd = pathToAdd.substring(deviceId.length());
		}
		
		// add the non common portion of the toPath
		relativePath += pathToAdd;
		plugin.log("relative path: " + relativePath);
		return relativePath;
	}	
	
	/**
	 * @return Returns an array of all of the files in the File from.
	 * If from is a file then an array containing only that file is returned.
	 * If from is a directory, then an array containing all the files in that
	 * directory is returned.  If the path is invalid or is a directory containing no
	 * files, then an empty array is returned.
	 */
	synchronized public static File[] getAll(File from){
		return getAll(from, null);
	}	

	/**
	 * @return Returns an array of all of the files in the File from.
	 * returns only files that match the given filter.  
	 * If filter is null all files are returned.
	 * If from is a file then an array containing only that file is returned.
	 * If from is a directory, then an array containing all the files in that
	 * directory is returned.  If the path is invalid or is a directory containing no
	 * files, then an empty array is returned.
	 */
	synchronized public static File[] getAll(File from, LoDFileFilter filter){
		try{
			// get list of files in this file
			File[] fileList;
			fileList = from.listFiles();
			Vector files = new Vector();
			
			// search list for directories and files
			for(int f = 0; f < fileList.length; f++){
				File thisFile = fileList[f];
				
				if(thisFile.isDirectory()){
					// add all the files in this directory
					files.addAll(Arrays.asList(getAll(thisFile, filter)));
					
				}else{
					if(filter == null || filter.accept(thisFile)){
						files.add(thisFile);
					}
				}
			}
			
			return (File[])files.toArray(new File[0]);
		}catch(Exception e){
			plugin.log("Failed to get all.");
			plugin.log("from: '"+from+"'");
			return new File[0];
		}
	}	

	/**
	 * @return returns the file from the default classpath with the given name
	 * or null if more than one or no files were found
	 */
	synchronized public static IFile getFileInDefaultClasspath(IProject project, String fileName){
		// find all files in the source folder that match the fileName
		IPath sourceFolder = Classpath.getDefaultClasspath(project);
		Vector filters = new Vector();
		filters.add(fileName);
		LoDFileFilter filter = new LoDFileFilter(filters);
		File[] files = Files.getAll(sourceFolder.makeAbsolute().toFile(), filter);
			
		// if only found one file then return it
		if(files.length == 1){
			Path path = new Path(files[0].getAbsolutePath());
			String relPath = Files.findRelativePath(project.getLocation(), path);
			return project.getFile(relPath);
		}else{
		// report failure
			plugin.log("failed to find: " + fileName + " in classpath of: " + fileName);
			plugin.log(" found " + files.length + " matching files");
			plugin.log(" files: " + Arrays.asList(files));
			return null;
		}
	}

}
